<?php

/**
 * webEdition CMS
 *
 * $Rev: b2e2950d4ecc $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2018-08-28 18:56 +0000 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_base_showDocument{

	private static function checkCustomerFilter(we_contents_root $we_doc){
		if(!$we_doc->documentCustomerFilter || isset($GLOBALS['getDocContentVersioning'])){
			return true;
		}

		// call session_start to init session, otherwise NO customer can exist
		if(!isset($_SESSION)){
			new we_base_sessionHandler();
		}

		if(($visitorHasAccess = $we_doc->documentCustomerFilter->accessForVisitor($we_doc->ID, $we_doc->ContentType))){
			$code = 0;
			switch($visitorHasAccess){
				case we_customer_documentFilter::ACCESS:
				case we_customer_documentFilter::CONTROLONTEMPLATE:
					return true;
				case we_customer_documentFilter::NO_LOGIN:
					$code = 401;
				//no break
				default:
					// user has NO ACCESS => show errordocument
					we_html_tools::setHttpCode($code ?: 403);
					$errorDocId = $we_doc->documentCustomerFilter->getErrorDoc($visitorHasAccess);
					if($errorDocId && ($errorDocPath = id_to_path($errorDocId, FILE_TABLE)) && file_exists($_SERVER['DOCUMENT_ROOT'] . $errorDocPath)){ // use given document instead !
						unset($errorDocId, $_REQUEST);
						include($_SERVER['DOCUMENT_ROOT'] . $errorDocPath);
						unset($errorDocPath);
						return false;
					}
					echo 'This document is not accessible';
					return false;
				//exit('Customer has no access to this document');
			}
		}
		return true;
	}

	public static function getWEDoc(){
//  Diese we_cmds werden auf den Seiten gespeichert und nicht übergeben!!!!!
//  Sie kommen von showDoc.php
		$we_ID = we_base_request::_(we_base_request::INT, 'we_cmd', 0, 1);
		$tmplID = we_base_request::_(we_base_request::INT, 'we_cmd', 0, 4);
//these come from external!
		$editmode = we_base_request::_(we_base_request::BOOL, 'we_cmd', false, 6);

		$we_Table = FILE_TABLE;

		$we_dt = isset($_SESSION['weS']['we_data'][$GLOBALS['we_transaction']]) ? $_SESSION['weS']['we_data'][$GLOBALS['we_transaction']] : [];

// init document
		$we_doc = we_contents_document::initDoc($we_dt, '', $we_ID, $we_Table);
		$cmd = we_base_request::_(we_base_request::STRING, 'cmd', '');
		switch($cmd){
			case '':
			case 'ResetVersion':
			case 'PublishDocs':
				break;
			default:
				if((!$we_doc->IsDynamic) && (!$tmplID)){ // if the document is not a dynamic php-doc and is published we make a redirect to the static page
					header('Location: ' . ($we_doc->Published ? $we_doc->Path : '/this_file_does_not_exist_on_this_server'));
					exit();
				}
		}
		//check if customer has access
		if(!self::checkCustomerFilter($we_doc)){
			return null;
		}
//FIXME: is this relevant at this point?!
		$we_doc->EditPageNr = $editmode ? we_base_constants::WE_EDITPAGE_CONTENT : we_base_constants::WE_EDITPAGE_PREVIEW;

		if($tmplID && ($we_doc->ContentType == we_base_ContentTypes::WEDOCUMENT)){ // if the document should displayed with an other template
			$we_doc->setTemplateID($tmplID);
		}
		return $we_doc;
	}

	public static function getWEDocObj(){
		$userID = (isset($GLOBALS['we_doc']) && $GLOBALS['we_doc']->InWebEdition ? $GLOBALS['we_doc']->isLockedByUser() : 0);

		if(($userID && $userID != $_SESSION['user']['ID']) || (we_base_request::_(we_base_request::STRING, 'we_cmd', '', 0) === 'switch_edit_page' || (isset($_SESSION['weS']['EditPageNr']) && $_SESSION['weS']['EditPageNr'] == we_base_constants::WE_EDITPAGE_PREVIEW))){ //	Preview-Mode of Tabs
			//	We must choose the right template to show the object.
			//	Therefore we must look, if $_SESSION['weS']['SEEM']['LASTPATH'] exists to check the workspace.
			//	First check the workspaces for the document.
			//	choose the template matching to the actual workspace.
			//	If wrong workspace or no template can be found, just show the name/value pairs.
			// init document
			if(!($we_transaction = we_base_request::_(we_base_request::TRANSACTION, 'we_transaction'))){
				exit();
			}
			$tid = 0;
			$we_dt = $_SESSION['weS']['we_data'][$we_transaction];

			// FIXME: to use the following (legacy) code from WE < 8 following fix is needed (#11583)
			// FIXME: set Templates as object property on init
			$GLOBALS['we_obj'] = $GLOBALS['we_doc'];
			$_SESSION['weS']['we_data'][$we_transaction][0]['Templates'] = $_SESSION['weS']['we_data'][$we_transaction][0]['classData']['Templates'];

			if(isset($_SESSION['weS']['we_data'][$we_transaction][0]['Templates'])){
				$tids = array_filter(explode(',', $_SESSION['weS']['we_data'][$we_transaction][0]['Templates'])); //	get all templateIds.
				$workspaces = makeArrayFromCSV($_SESSION['weS']['we_data'][$we_transaction][0]['Workspaces']);

				$tmpDB = new DB_WE();

				//	determine Path from last opened wE-Document
				$lastDoc = isset($_SESSION['weS']['last_webEdition_document']) ? $_SESSION['weS']['last_webEdition_document'] : [];
				if(!empty($lastDoc['Path']) && $workspaces){// get the correct template
					$paths = id_to_path($workspaces, FILE_TABLE, $tmpDB, true);
					//	Select a matching workspace.
					foreach($paths as $workspace){

						if(strpos($lastDoc['Path'], $workspace) === 0 && $tids){
							//	init document
							$tid = $tids[0];
							break;
						}
					}
				}

				if(!$tid){
					if(($paths = id_to_path(array_filter($tids), TEMPLATES_TABLE, $tmpDB, true))){
						$tid = key($paths);
					}
				}
				unset($tmpDB);
			}

			if($tid){
				//	init document
				$GLOBALS['we_doc']->we_initSessDat($we_dt);
				$_REQUEST[WEOBJ_PARAM] = $_SESSION['weS']['we_data'][$we_transaction][0]['ID'];
			} else {
				self::showContent($GLOBALS['we_doc']);
				exit;
			}
		} else if(($we_transaction = we_base_request::_(we_base_request::TRANSACTION, 'we_cmd', '', 3))){

			$tid = we_base_request::_(we_base_request::INT, 'we_cmd', 0, 2);
			// init document
			$we_dt = $_SESSION['weS']['we_data'][$we_transaction];
			$GLOBALS['we_doc'] = we_contents_document::initDoc($we_dt);
		} else { //	view with template
			$tid = 0;
			if(($oid = we_base_request::_(we_base_request::INT, WEOBJ_PARAM, 0))){
				$GLOBALS['we_obj'] = new we_objectFile();
				$GLOBALS['we_obj']->initByID($oid, OBJECT_FILES_TABLE);
				$GLOBALS['we_obj']->setTitleAndDescription();
			}

			if(!$oid || !$GLOBALS['we_obj']->Published){
				we_html_tools::setHttpCode(404);

				if(ERROR_DOCUMENT_NO_OBJECTFILE && ($path = id_to_path(ERROR_DOCUMENT_NO_OBJECTFILE, FILE_TABLE))){
					//if set, we show object again!
					unset($_REQUEST);
					include($_SERVER['DOCUMENT_ROOT'] . $path);
					exit();
				}
				echo 'Sorry, we are unable to locate your requested Page.';
				exit();
			}

			$GLOBALS['we_doc'] = new we_webEditionDocument();
			$GLOBALS['we_doc']->initByID(we_base_request::_(we_base_request::INT, 'we_cmd', 0, 1), FILE_TABLE);
			$GLOBALS['we_doc']->initByObj($GLOBALS['we_obj']);
		}
		//check if customer has access
		if(empty($GLOBALS['we_obj']) || !self::checkCustomerFilter($GLOBALS['we_obj'])){
			return null;
		}

		return $tid;
	}

	private static function setBuffer(we_contents_root $we_doc){
		if(empty($GLOBALS['we']['Charset']['Page']) && ($cs = $we_doc->getElement('Charset', we_contents_base::ETYPE_TEXT))){ //	send charset which might be determined in template
			$GLOBALS['we']['Charset']['Page'] = $cs;
			we_html_tools::headerCtCharset('text/html', $cs, true);
		}

		$GLOBALS['we']['urlReplace'] = empty($GLOBALS['we']['urlReplace']) ? we_folder::getUrlReplacements($GLOBALS['DB_WE']) : $GLOBALS['we']['urlReplace'];
// --> Glossary Replacement
		$GLOBALS['we']['useGlossary'] = empty($GLOBALS['we']['useGlossary']) ? ((we_modules_info::isActive(we_modules_info::GLOSSARY) && (empty($GLOBALS['we']['main']['id']) || $GLOBALS['we']['main']['id'] == $GLOBALS['we_doc']->ID)) && (isset($we_doc->InGlossar) && $we_doc->InGlossar == 0) && we_glossary_replace::useAutomatic()) : $GLOBALS['we']['useGlossary'];
		$useBuffer = !empty($GLOBALS['we']['urlReplace']) || $GLOBALS['we']['useGlossary'];

		if($useBuffer){
			ob_start();
		}
	}

	public static function prePareInc(we_contents_root $we_doc){
		if(!($we_include = $we_doc->editor())){
			return false;
		}
		if(substr(strtolower($we_include), 0, strlen($_SERVER['DOCUMENT_ROOT'])) == strtolower($_SERVER['DOCUMENT_ROOT'])){
			self::setBuffer($we_doc);
			return $we_include;
		}
		we_html_tools::protect(); //	only inside webEdition !!!
		return WE_INCLUDES_PATH . $we_include;
	}

	public static function prePareIncObj(we_objectFile $we_obj, $tid){
		//if we don't have a tid yet, we get it from the workspace - we_cmd[1] is set in the dynamic document
		$tid = $tid ?: $we_obj->getTemplateID(f('SELECT ParentID FROM ' . FILE_TABLE . ' WHERE ID=' . we_base_request::_(we_base_request::INT, 'we_cmd', 0, 1)));
		//f('SELECT ParentID FROM ' . FILE_TABLE . ' WHERE Path=CONCAT(COALESCE((SELECT Path FROM ' . FILE_TABLE . ' WHERE urlMap LIKE "' . $_SERVER['SERVER_NAME'] . '%" LIMIT 1),""),"' . $GLOBALS['DB_WE']->escape(getScriptName()) . '")')

		$tmplPath = $tid ? preg_replace('/.tmpl$/i', '.php', f('SELECT Path FROM ' . TEMPLATES_TABLE . ' WHERE ID=' . intval($tid))) : '';
		if(!$tid || !$tmplPath || !is_readable(TEMPLATES_PATH . $tmplPath)){
			we_html_tools::setHttpCode(SUPPRESS404CODE ? 200 : 404);

			if(ERROR_DOCUMENT_NO_OBJECTFILE && ($path = id_to_path(ERROR_DOCUMENT_NO_OBJECTFILE, FILE_TABLE))){
				//if set, we show object again!
				unset($_REQUEST);
				include($_SERVER['DOCUMENT_ROOT'] . $path);
				exit();
			}
			return '';
		}
		if(empty($GLOBALS['we']['Charset']['Page']) && ($cs = $GLOBALS['we_doc']->getElement('Charset', we_contents_base::ETYPE_TEXT))){ //	send charset which might be determined in template
			$GLOBALS['we']['Charset']['Page'] = $cs;
			we_html_tools::headerCtCharset('text/html', $cs, true);
		}
		if(!$GLOBALS['we_doc']->InWebEdition){
			self::setBuffer($GLOBALS['we_doc']);
		}
		return TEMPLATES_PATH . $tmplPath;
	}

	public static function postInc(we_contents_root $we_doc){
		$useBuffer = !empty($GLOBALS['we']['urlReplace']) || !empty($GLOBALS['we']['useGlossary']);
		if(!$useBuffer){
			return;
		}
		$content = ob_get_clean();

		if($GLOBALS['we']['useGlossary']){
			$content = we_glossary_replace::replace($content, $GLOBALS['we_doc']->Language);
		}
		if($GLOBALS['we']['urlReplace']){
			$content = preg_replace($GLOBALS['we']['urlReplace'], array_keys($GLOBALS['we']['urlReplace']), $content);
		}
		echo $content;
	}

	/**
	 * showContent()
	 * @desc     Generates and prints a default template to see the attribs of an object.
	 *
	 * @param    msg		string
	 */
	private static function showContent(we_objectFile $we_obj){
		$editor = new we_editor_contentobjectFile($we_obj);
		$editor->setPreview();
		echo $editor->show();
		exit;
	}

}
