<?php

/**
 * webEdition CMS
 *
 * $Rev: 898571e5dc62 $
 * $Author: mokraemer $
 * $Date: 2018-11-29 18:52 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_captcha_captcha{

	/**
	 * display the image
	 *
	 * @return string
	 */
	public static function display(we_captcha_image $image, $type = "gif"){
		list($type, $data) = self::get($image, $type);

		header('Expires: ' . gmdate('D, d M Y H:i:s') . ' GMT');
		header('Cache-Control: no-cache, must-revalidate, max-age=0');
		header('Pragma: no-cache');
		header('Content-type: ' . $type);
		return $data;
	}

	public static function get(we_captcha_image $image, $type = "gif"){
		$code = '';
		$im = $image->get($code);
		// save the code to the memory
		self::save($code);

		ob_start();
		$ctype = '';
		switch($type){
			case 'jpg':
				$ctype = 'image/jpeg';
				imagejpeg($im);
				imagedestroy($im);
				break;
			case 'png':
				$ctype = 'image/png';
				imagepng($im);
				imagedestroy($im);
				break;
			case 'gif':
			default:
				$ctype = 'image/gif';
				imagegif($im);
				imagedestroy($im);
				break;
		}
		return [$ctype, ob_get_clean()];
	}

	public static function check($captcha, $type = 'captcha', $strict = false){
		static $valid = [];
		if(isset($valid[$captcha])){
			return true;
		}
		$db = $GLOBALS['DB_WE'];
		self::cleanup();
//don't check IP, since IPv4/IPv6 switches between requests
//IP=x\'' . bin2hex(inet_pton(strstr($_SERVER['REMOTE_ADDR'], ':') ? $_SERVER['REMOTE_ADDR'] : '::ffff:' . $_SERVER['REMOTE_ADDR'])) . '\' AND
		$db->query('DELETE FROM ' . CAPTCHA_TABLE . ' WHERE typ="' . $db->escape($type) . '" AND BINARY code="' . $db->escape($captcha) . '"' . ($strict ? ' AND agent=x\'' . $db->escape(md5($_SERVER['HTTP_USER_AGENT']), true) . '\'' : ''), '', $db);
		if($db->affected_rows()){
			$valid[$captcha] = true;
			return true;
		}
		return false;
	}

	public static function cleanup(){
		$GLOBALS['DB_WE']->query('DELETE FROM ' . CAPTCHA_TABLE . ' WHERE valid<NOW()');
	}

	public static function hasTopt($totp, $maxValid = 180){
		self::cleanup();
		if(!f('SELECT 1 FROM ' . CAPTCHA_TABLE . ' WHERE typ="totp" AND code="' . $GLOBALS['DB_WE']->escape($totp) . '"')){
			//token not present, add token, so it can't be used for the next 3 minutes
			self::save($totp, 'totp', $maxValid);
			return false;
		}
		//token found, login must be blocked!
		return true;
	}

	/**
	 * Save the Captcha Code
	 *
	 * @param string $captcha
	 * @return void
	 */
	public static function save($captcha, $type = 'captcha', $validity = 1800){
		$db = $GLOBALS['DB_WE'];
		self::cleanup();
//FIMXE: make IP bin save
		$db->query('REPLACE INTO ' . CAPTCHA_TABLE . ' SET ' . we_database_base::arraySetter([
				'IP' => inet_pton(strstr($_SERVER['REMOTE_ADDR'], ':') ? $_SERVER['REMOTE_ADDR'] : '::ffff:' . $_SERVER['REMOTE_ADDR']),
				'agent' => empty($_SERVER['HTTP_USER_AGENT']) ? '' : sql_function('x\'' . md5($_SERVER['HTTP_USER_AGENT']) . '\''),
				'typ' => $type,
				'code' => $captcha,
				'valid' => sql_function('NOW()+INTERVAL ' . $validity . ' SECOND'),
		]));
	}

}
