<?php

/**
 * webEdition CMS
 *
 * $Rev: 7a28c72dbff9 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2018-09-12 11:47 +0000 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_chooser_multiDirTemplateAndDefault extends we_chooser_multiDir{
	private $defaultName = '';
	private $defaultArr = [];
	var $tmplcsv = '';
	var $tmplSelectName = '';
	var $mustTemplateIDs = '';
	var $tmplArr = '';
	var $tmplVals = [];
	var $mustTemplateIDsArr;
	var $mustPaths;
	var $create = 0;

	public function __construct($width, $ids, $cmd_del, $addbut, $ws = '', $tmplcsv = '', $tmplSelectName = '', $mustTemplateIDs = '', $tmplWs = '', $defaultName = '', $defaultCSV = '', $fields = 'Path', $table = FILE_TABLE, $css = ''){
		$this->defaultName = $defaultName;
		$this->defaultArr = $defaultCSV === '' ? [] : explode(',', $defaultCSV);
		parent::__construct($width, $ids, $cmd_del, $addbut, $ws, $fields, $table, $css);
		$this->tmplcsv = $tmplcsv;
		$this->tmplSelectName = $tmplSelectName;
		$this->mustTemplateIDs = $mustTemplateIDs;
		$this->mustTemplateIDsArr = array_filter(explode(',', $this->mustTemplateIDs));
		$this->tmplArr = array_filter(explode(',', $this->tmplcsv));
		$this->tmplValsArr = getPathsFromTable(TEMPLATES_TABLE, $this->db, we_base_constants::FILE_ONLY, get_ws(TEMPLATES_TABLE), 'Path');
		$this->mustPaths = id_to_path($this->mustTemplateIDsArr, TEMPLATES_TABLE, $this->db, true);
		foreach($this->mustTemplateIDsArr as $i => $id){
			if(!in_array($id, array_keys($this->tmplValsArr))){
				$this->tmplValsArr[$id] = isset($this->mustPaths[$i]) ? $this->mustPaths[$i] : "";
			}
		}
		$this->lines = 3;
	}

	function getRootLine($lineNr){

		switch($lineNr){
			case 0:
				return '<tr>
	<td class="chooserFileIcon" data-contenttype="' . we_base_ContentTypes::FOLDER . '"></td>
	<td class="' . $this->css . '">/</td>
	<td>' . ((($this->isEditable && $this->cmd_del) || $this->CanDelete) ?
					we_html_button::create_button(we_html_button::TRASH, "javascript:" . $this->getJsSetHot() . ($this->extraDelFn ?: "") . ";we_cmd('" . $this->cmd_del . "','0');") :
					"") . '</td>
</tr>';
			default:
				return $this->getLine($lineNr);
		}
	}

	private function getTemplateSelector($idname, $id, $path){
		$weSuggest = & we_gui_suggest::getInstance();
		$textname = $idname . 'Name';

		$weSuggest->setAcId('Template');
		$weSuggest->setContentType([we_base_ContentTypes::TEMPLATE]);
		$weSuggest->setInput($textname, $path);
		$weSuggest->setMaxResults(10);
		$weSuggest->setResult($idname, $id);
		$weSuggest->setSelector(we_gui_suggest::DocSelector);
		$weSuggest->setTable(TEMPLATES_TABLE);
		$weSuggest->setWidth(0);
		$weSuggest->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',document.we_form.elements['" . $idname . "'].value,WE().consts.tables.TEMPLATES_TABLE,'" . $idname . "','" . $textname . "','setHot','','',WE().consts.contentTypes.TEMPLATE,1)"));
		//$weSuggest->setIsDropFromTree(true);// deactivated
		return $weSuggest->getHTML();
	}

	function getLine($lineNr){

		//$editable = $this->isEditable();
		switch($lineNr){
			case 0:
				return parent::getLine(0);
			case 1:
				$idArr = explode(',', $this->ids);
				$checkbox = we_html_forms::checkbox($idArr[$this->nr], (in_array($idArr[$this->nr], $this->defaultArr) ? true : false), $this->defaultName . "_" . $this->nr, g_l('weClass', '[standard_workspace]'));
				return '<tr><td></td><td>' . $checkbox . '</td><td style="width:50px;"></td></tr>';
			case 2:
				if($this->create){
					$but = we_html_button::create_button('fa:btn_add_template,fa-plus,fa-lg fa-file-code-o', "javascript:we_cmd('object_create_tmpfromClass','0','" . $this->nr . "','" . $GLOBALS['we_transaction'] . "')");
				} else {
					$but = we_html_button::create_button(we_html_button::VIEW, "javascript:we_cmd('object_preview_objectFile','0','" . (isset($this->tmplArr[$this->nr]) ? $this->tmplArr[$this->nr] : "") . "','" . $GLOBALS['we_transaction'] . "')");
				}
				$path = id_to_path(isset($this->tmplArr[$this->nr]) ? $this->tmplArr[$this->nr] : "", TEMPLATES_TABLE, $this->db);
				if($this->isEditable){
					return '<tr><td></td><td>' . we_html_element::span(['class' => "small"], we_html_element::b(g_l('weClass', '[template]') . ':')) . we_html_element::br() . $this->getTemplateSelector($this->tmplSelectName . "_" . $this->nr, isset($this->tmplArr[$this->nr]) ? $this->tmplArr[$this->nr] : 0, $path) . '</td><td style="vertical-align:bottom">' . $but . '</td></tr>';
				}
				return '<tr><td></td><td class="' . $this->css . '">' . we_html_element::span(['class' => "small"], we_html_element::b(g_l('weClass', '[template]') . ':')) . we_html_element::br() . $path . we_html_element::hidden($this->tmplSelectName . "_" . $this->nr, (isset($this->tmplArr[$this->nr]) ? $this->tmplArr[$this->nr] : "")) . '" /></td><td style="vertical-align:bottom">' . $but . '</td></tr>';
			default:
				return '';
		}
	}

}
