<?php
/**
 * webEdition CMS
 *
 * $Rev: 4162fe225a67 $
 * $Author: Marc Krämer <marc.kraemer@aldente-it.de> $
 * $Date: 2019-03-27 13:22 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @internal
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
ini_set('mysqli.reconnect', 1);
!defined('MYSQL_BOTH') && define('MYSQL_BOTH', MYSQLI_BOTH);
!defined('MYSQL_ASSOC') && define('MYSQL_ASSOC', MYSQLI_ASSOC);
!defined('MYSQL_NUM') && define('MYSQL_NUM', MYSQLI_NUM);

class DB_WE extends we_database_base{

	protected function _free(){
		if(is_object($this->Query_ID)){
//			print_r(debug_backtrace());
			@$this->Query_ID->free();
		}
	}

	protected function _query($Query_String, $unbuffered = false){
		$this->_free();
		$tmp = $this->Link_ID->query($Query_String, ($unbuffered ? MYSQLI_USE_RESULT : MYSQLI_STORE_RESULT));
		if($tmp === false){
			return 0;
		}
		if($tmp === true){
			return -1; //!=0 what is tested.
		}
		return $tmp;
	}

	protected function _setCharset($charset){
		$this->Link_ID->set_charset($charset);
	}

	protected function _seek($pos = 0){
		return (is_object($this->Query_ID)) && $this->Query_ID->data_seek($pos);
	}

	protected function setError(){
		$this->Warnings = [];
		if($this->Link_ID){
			$this->Errno = $this->Link_ID->errno;
			$this->Error = $this->Link_ID->error;
			if($this->Link_ID->warning_count){
				$warn = $this->Link_ID->get_warnings();
				for($i = 0; $i < $this->Link_ID->warning_count; $i++){
					$this->Warnings[] = $warn->message;
				}
			}
		} else {
			$this->Errno = 2006;
			$this->Error = '';
		}
	}

	protected function info(){
		return ($this->Link_ID ? $this->Link_ID->info : '');
	}

	protected function fetch_array($resultType){
		return (is_object($this->Query_ID)) ?
			$this->Query_ID->fetch_array($resultType) :
			false;
	}

	protected function _affected_rows(){
		return $this->Link_ID->affected_rows;
	}

	public function close(){
		if($this->Link_ID){
			$this->Link_ID->close();
			$this->Link_ID = null;
			$this->Query_ID = null;
		}
	}

	protected function connect($Database = DB_DATABASE, $Host = DB_HOST, $User = DB_USER, $Password = DB_PASSWORD){
		$this->Database = $Database;
		if($this->isConnected()){
			return true;
		}
		$hostPort = explode(':', $Host);
		$Host = $hostPort[0];
		$Port = isset($hostPort[1]) ? $hostPort[1] : null;

		switch(DB_CONNECT){
			case 'pconnect'://old mysql if
			case 'mysqli_pconnect':
				$Host = 'p:' . $Host;
			case 'connect'://old mysql if
			case 'mysqli_connect':
				$this->Query_ID = null;
				$this->Link_ID = mysqli_init();
				$this->Link_ID->options(MYSQLI_OPT_CONNECT_TIMEOUT, 60);
				if(defined('MYSQLI_OPT_INT_AND_FLOAT_NATIVE')){
					$this->Link_ID->options(MYSQLI_OPT_INT_AND_FLOAT_NATIVE, 1);
				}
				if((!@$this->Link_ID->real_connect($Host, $User, $Password, $Database, $Port, null, MYSQLI_CLIENT_COMPRESS) &&
					!@$this->Link_ID->real_connect($Host, $User, $Password, $Database, $Port, null, MYSQLI_CLIENT_COMPRESS) &&
					!@$this->Link_ID->real_connect($Host, $User, $Password, $Database, $Port, null, MYSQLI_CLIENT_COMPRESS)

					) ||
					//need the @ operator, since can't catch mysqli warning on reconnect pconnection
					$this->Link_ID->connect_error){
					$this->Link_ID = null;
					$GLOBALS['we']['ERROR'] = 'mysqli_(p)connect(' . $Host . ',' . $User . ') ' . ($this->Link_ID ? $this->Link_ID->connect_error : '') . ' failed.';
					//trigger_error('WE-Connect error ' . $this->Link_ID->connect_error, E_USER_WARNING);
					//$this->halt("mysqli_(p)connect($Host, $User) failed.");
					return false;
				}
				break;
			default:
				$GLOBALS['we']['ERROR'] = 'Error in DB connect';
				return false;
		}
		$this->_setup();
		return true;
	}

	/**
	 * call this function to make sure the connection is setup correctly
	 */
	protected function _setup(){
// deactivate MySQL strict mode; don't use query function (error logging)
		//FIXME: testing for more errors in WE
		$this->_query('SET SESSION sql_mode="ERROR_FOR_DIVISION_BY_ZERO,NO_ENGINE_SUBSTITUTION,NO_ZERO_DATE,NO_ZERO_IN_DATE"'); //NO_ZERO_DATE,NO_ZERO_IN_DATE,
		if(defined('DB_SET_CHARSET') && DB_SET_CHARSET != ''){
			$this->_setCharset(DB_SET_CHARSET);
		}
	}

	public function field_flags($no){
		return (is_object($this->Query_ID) ? $this->Query_ID->fetch_field_direct($no)->flags : '');
	}

	public function field_len($no){
		if(is_object($this->Query_ID)){
			$len = $this->Query_ID->fetch_field_direct($no)->length;
			//fix faulty lenght on text-types with connection in utf-8
			$type = $this->Query_ID->fetch_field_direct($no)->type;
			if(DB_SET_CHARSET === 'utf8' && $type >= 252 && $type <= 254){
				$len /= 3;
			}
			return $len;
		} else {
			return 0;
		}
	}

	public function field_name($no){
		return (is_object($this->Query_ID) ? $this->Query_ID->fetch_field_direct($no)->name : '');
	}

	public function field_table($no){
		return (is_object($this->Query_ID) ? $this->Query_ID->fetch_field_direct($no)->table : '');
	}

	public function field_type($no){
		switch(is_object($this->Query_ID) ? $this->Query_ID->fetch_field_direct($no)->type : 0){
			case MYSQLI_TYPE_CHAR:
				return self::TYPE_TINYINT;
			case MYSQLI_TYPE_SHORT:
				return self::TYPE_SMALLINT;
			case MYSQLI_TYPE_LONG:
				return self::TYPE_INT;
			case MYSQLI_TYPE_INT24:
				return self::TYPE_MEDIUMINT;
			case MYSQLI_TYPE_LONGLONG:
				return self::TYPE_BIGINT;
			case MYSQLI_TYPE_FLOAT:
				return self::TYPE_FLOAT;
			case MYSQLI_TYPE_DOUBLE:
				return self::TYPE_DOUBLE;
			case MYSQLI_TYPE_NEWDECIMAL:
				return self::TYPE_DECIMAL;
			case MYSQLI_TYPE_NULL:
				return self::TYPE_NULL;
			case MYSQLI_TYPE_TIMESTAMP:
				return self::TYPE_TIMESTAMP;
			case MYSQLI_TYPE_DATE:
				return self::TYPE_DATE;
			case MYSQLI_TYPE_TIME:
				return self::TYPE_TIME;
			case MYSQLI_TYPE_DATETIME:
				return self::TYPE_DATETIME;
			case MYSQLI_TYPE_YEAR:
				return self::TYPE_YEAR;
			case MYSQLI_TYPE_NEWDATE:
				return self::TYPE_DATE;
			case MYSQLI_TYPE_BIT:
				return self::TYPE_BIT;
			case MYSQLI_TYPE_ENUM:
				return self::TYPE_ENUM;
			case MYSQLI_TYPE_SET:
				return self::TYPE_SET;
			case MYSQLI_TYPE_TINY_BLOB:
				return ($this->field_flags($no) & MYSQLI_BINARY_FLAG ? self::TYPE_TINYBLOB : self::TYPE_TINYTEXT);
			case MYSQLI_TYPE_MEDIUM_BLOB:
				return ($this->field_flags($no) & MYSQLI_BINARY_FLAG ? self::TYPE_MEDIUMBLOB : self::TYPE_MEDIUMTEXT);
			case MYSQLI_TYPE_BLOB:
				return ($this->field_flags($no) & MYSQLI_BINARY_FLAG ? self::TYPE_BLOB : self::TYPE_TEXT);
			case MYSQLI_TYPE_LONG_BLOB:
				return ($this->field_flags($no) & MYSQLI_BINARY_FLAG ? self::TYPE_LONGBLOB : self::TYPE_LONGTEXT);
			case MYSQLI_TYPE_GEOMETRY:
				return self::TYPE_GEOMETRY;
			//252 is currently mapped to all text and blob types (MySQL 5.0.51a)
			case MYSQLI_TYPE_VAR_STRING:
				return ($this->field_flags($no) & MYSQLI_BINARY_FLAG ? self::TYPE_VARBIN : self::TYPE_VARCHAR);
			case MYSQLI_TYPE_STRING:
				return ($this->field_flags($no) & MYSQLI_BINARY_FLAG ? self::TYPE_BIN : self::TYPE_CHAR);
			default:
				return '';
		}
	}

	public function field_type_short($no){
		switch(is_object($this->Query_ID) ? $this->Query_ID->fetch_field_direct($no)->type : 0){
			case MYSQLI_TYPE_CHAR:
			case MYSQLI_TYPE_SHORT:
			case MYSQLI_TYPE_LONG:
			case MYSQLI_TYPE_INT24:
			case MYSQLI_TYPE_LONGLONG:
			case MYSQLI_TYPE_FLOAT:
			case MYSQLI_TYPE_DOUBLE:
			case MYSQLI_TYPE_NEWDECIMAL:
			case MYSQLI_TYPE_YEAR:
			case MYSQLI_TYPE_BIT:
				return self::TYPE_SHORT_NUMBER;
			case MYSQLI_TYPE_NULL:
				return self::TYPE_NULL;
			case MYSQLI_TYPE_TIMESTAMP:
			case MYSQLI_TYPE_DATE:
			case MYSQLI_TYPE_TIME:
			case MYSQLI_TYPE_DATETIME:
			case MYSQLI_TYPE_NEWDATE:
				return self::TYPE_SHORT_DATE;
			case MYSQLI_TYPE_ENUM:
			case MYSQLI_TYPE_SET:
			case MYSQLI_TYPE_TINY_BLOB:
			case MYSQLI_TYPE_MEDIUM_BLOB:
			case MYSQLI_TYPE_BLOB:
			case MYSQLI_TYPE_LONG_BLOB:
			//252 is currently mapped to all text and blob types (MySQL 5.0.51a)
			case MYSQLI_TYPE_VAR_STRING:
			case MYSQLI_TYPE_STRING:
				return self::TYPE_SHORT_STRING;
			case MYSQLI_TYPE_GEOMETRY:
				return self::TYPE_GEOMETRY;
			default:
				return '';
		}
	}

	protected function _getInsertId(){
		return $this->Link_ID->insert_id;
	}

	public function num_fields(){
		return (is_object($this->Query_ID) ? $this->Query_ID->field_count : 0);
	}

	public function num_rows(){
		return is_object($this->Query_ID) ? $this->Query_ID->num_rows : 0;
	}

	public function _getCurrentCharset(){
		$charset = mysqli_get_charset($this->Link_ID);
		return $charset->charset;
	}

	public function _getInfo(){
		$charset = mysqli_get_charset($this->Link_ID);
		return [
			'type' => DB_CONNECT,
			'protocol' => $this->Link_ID->protocol_version,
			'client' => $this->Link_ID->client_info,
			'host' => $this->Link_ID->host_info,
			'server' => $this->Link_ID->server_info,
			'database' => $this->Database,
			'encoding' => $charset->charset
		];
	}

	protected function ping(){
		return $this->Link_ID->ping();
	}

	public function fieldNames(){
		if(!($this->Query_ID) || !is_object($this->Query_ID)){
			return [];
		}
		$fields = $this->Query_ID->fetch_fields();
		$res = [];
		foreach($fields as $f){
			$res[] = $f->name;
		}
		return $res;
	}

}
