<?php

/**
 * webEdition CMS
 *
 * $Rev: 03a01cfada9e $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2018-08-21 16:22 +0000 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_dialog_base{
	/*	 * ***********************************************************************
	 * VARIABLES
	 * *********************************************************************** */
	protected $db = '';
	protected $what = '';
	protected $args = [];
	protected $dialogTitle = '';
	protected $changeableArgs = [];
	protected $pageNr = 1;
	protected $numPages = 1;
	protected $onOkJsOnly = false;
	protected $charset = '';
	protected $bodyId = 'weDialog';
	protected $noInternals = true;
	protected $we_cmd = [];
	protected $jsCmd = null;
	protected $tinyDialogName = '';
	protected $isTinyDialog = false;

	/**
	 * Constructor of class.
	 *
	 */
	protected function __construct($noInternals = true){
		$this->noInternals = $noInternals;
		$this->jsCmd = new we_base_jsCmd();
		$this->db = new DB_WE();
	}

	protected function initByHttp(){
		$this->what = we_base_request::_(we_base_request::STRING, 'we_what', '');
		$this->we_cmd = we_base_request::_(we_base_request::STRING, 'we_cmd', []);
		if(($args = we_base_request::_(we_base_request::STRING, 'we_dialog_args'))){//assume no tags are allowed
			$this->args = $args;
			foreach($this->args as &$value){
				$value = urldecode($value);
			}
		}

		$this->pageNr = we_base_request::_(we_base_request::INT, 'we_pageNr', $this->pageNr);
	}

	protected function getHTML(){
		if($this->onOkJsOnly){
			$this->what = 'dialog';
		}

		switch($this->what){
			case 'cmd':
				return $this->getCmdHTML();
			default:
				return $this->getHeaderHTML('', $this->getFramesetHTML(), $this->isTinyDialog);
		}
	}

	public static function getDialog($noInternals = true){
		$inst = new static($noInternals);

		return $inst->getHTML();
	}

	protected function getCmdHTML(){
		$send = [];

		// " quote for correct work within ""
		foreach($this->args as $k => $v){
			$send[$k] = str_replace('"', '\"', $v);
		}

		return $this->cmdFunction($send) . $this->jsCmd->getCmds();
	}

	protected function cmdFunction(array $args){
		// overwrite
		return '';
	}

	protected function getFramesetHTML(){
		return we_html_element::body(['onload' => 'weDialogBase_setLoaded();', 'id' => $this->bodyId, 'class' => 'weDialogBody', 'onunload' => 'doUnload()'], we_html_element::exIFrame('main', $this->getDialogHTML(), 'position:absolute;top:0;bottom:0;left:0;right:0;overflow: hidden;') .
				we_html_element::iframe('we_cmd_frame', 'about:blank', 'position:absolute;height:0;bottom:0;left:0;right:0;overflow: hidden;') .
				$this->getBodyEndpart()
		);
	}

	protected function getBodyEndpart(){
		// to be overwritten
		return '';
	}

	protected function getNextBut(){
		return we_html_button::create_button(we_html_button::NEXT, "javascript:document.forms['0'].submit();");
	}

	protected function getOkBut(){
		return we_html_button::create_button(we_html_button::OK, 'javascript:weDoOk();');
	}

	protected function getCancelBut(){
		return we_html_button::create_button(we_html_button::CANCEL, 'javascript:top.close();');
	}

	protected function getbackBut(){
		return ($this->pageNr > 1) ? we_html_button::create_button(we_html_button::BACK, 'javascript:history.back();') : '';
	}

	protected function getDialogHTML(){
		$dc = $this->getDialogContentHTML();

		$dialogContent = (is_array($dc) ?
			we_html_multiIconBox::getHTML('', $dc, true, $this->getDialogButtons(), -1, '', '', false, $this->dialogTitle) :
			we_html_tools::dialogLayout($dc, $this->dialogTitle, $this->getDialogButtons()));

		return $this->getFormHTML() . $dialogContent .
			we_html_element::hidden('we_what', "cmd") . $this->getHiddenArgs() . '</form>';
	}

	protected function getDialogButtons(){
		if($this->pageNr == $this->numPages && $this->onOkJsOnly == false){
			$back = $this->getBackBut();
			$ok = we_html_button::create_button(we_html_button::OK, we_html_button::WE_FORM . ':we_form');
			$okBut = $back ? $back . $ok : $ok;
		} else if($this->pageNr < $this->numPages){
			$back = $this->getBackBut();
			$next = $this->getNextBut();
			$okBut = ($back && $next ) ? $back . $next : ($back ?: $next );
		} else {
			$back = $this->getBackBut();
			$ok = $this->getOkBut();
			$okBut = ($back && $ok ) ? $back . $ok : ($back ?: $ok );
		}
		return we_html_button::position_yes_no_cancel($okBut, '', $this->getCancelBut());
	}

	protected function getFormHTML(){
		$hiddens = '';
		if(($cmd = we_base_request::_(we_base_request::STRING, 'we_cmd'))){
			foreach($cmd as $k => $v){
				//TODO: why should we loop this commands through?
				$hiddens .= '<input type="hidden" name="we_cmd[' . $k . ']" value="' . rawurlencode($v) . '"/>';
			}
		}

		//create some empty we_cmds to be filled by JS if needed
		for($i = 0; $i < 4; $i++){
			$hiddens .= isset($_REQUEST['we_cmd'][$i]) ? '' : we_html_element::hidden("we_cmd[$i]", '');
		}

		$target = (!$this->onOkJsOnly ? ' target="we_cmd_frame"' : '');

		return '<form name="we_form" action="' . getScriptName() . '" method="post"' . $target . '>' . $hiddens;
	}

	protected function getHiddenArgs(){
		$hiddenArgs = '';

		foreach($this->args as $k => $v){
			if(!in_array($k, $this->changeableArgs)){
				$hiddenArgs .= we_html_element::hidden('we_dialog_args[' . $k . ']', $v);
			}
		}
		return $hiddenArgs;
	}

	protected function getDialogContentHTML(){
		return ''; // overwrite !
	}

	protected function getHeaderHTML($additionals = '', $body = ''){
		return we_html_tools::getHtmlTop($this->dialogTitle, $this->charset, '', we_html_element::jsScript(JS_DIR . 'dialogs/we_dialog_base.js', '', [
					'id' => 'loadVarDialog',
					'data-vars' => setDynamicVar([
						'onEnterKey' => intval($this->pageNr == $this->numPages && $this->onOkJsOnly),
						'pluginName' => $this->tinyDialogName
				])]) .
				($this->isTinyDialog ? self::getTinyMceDialogJS() .
				we_html_element::cssLink(CSS_DIR . 'wysiwyg/tinymce/weDialogCss.css') : ''
				) .
				$this->getJs() .
				$additionals .
				$this->jsCmd->getCmds(), $body);
	}

	protected static function getTinyMceDialogJS(){
		return we_html_element::jsScript(TINYMCE_SRC_DIR . 'tiny_mce_popup.js') .
			(IS_TINYMCE_4 ? we_html_element::jsScript(TINYMCE_SRC_DIR . 'weTinyMce_popup_overwrites.js') : '') .
			we_html_element::jsScript(TINYMCE_SRC_DIR . 'utils/mctabs.js') .
			we_html_element::jsScript(TINYMCE_SRC_DIR . 'utils/form_utils.js') .
			we_html_element::jsScript(TINYMCE_SRC_DIR . 'utils/validate.js') .
			we_html_element::jsScript(TINYMCE_SRC_DIR . 'utils/editable_selects.js');
	}

	protected function getJs(){
		return '';
	}

	protected function getHttpVar($type, $name, $alt = ''){
		return we_base_request::_($type, 'we_dialog_args', $alt, $name);
	}

	protected function getLangField($name, $title, $width){
		//FIXME: these values should be obtained from global settings
		$foo = we_html_tools::textInput('we_dialog_args[' . $name . ']', (isset($this->args[$name]) ? $this->args[$name] : ''), '', [], "text", $width - 50);
		$foo2 = '<select style="width:50px;" name="' . $name . '_select" onchange="this.form.elements[\'we_dialog_args[' . $name . ']\'].value=this.options[this.selectedIndex].value;this.selectedIndex=-1;">
	<option value=""></option>
	<option value="en">en</option>
	<option value="de">de</option>
	<option value="es">es</option>
	<option value="fi">fi</option>
	<option value="ru">ru</option>
	<option value="fr">fr</option>
	<option value="nl">nl</option>
	<option value="pl">pl</option>
</select>';
		return we_html_tools::formElementTable($title, $foo, $foo2);
	}

	protected function getClassSelect($style = 'width: 300px;'){
		$clSelect = new we_html_select([
			'name' => 'we_dialog_args[cssclass]',
			'id' => 'we_dialog_args[cssclass]',
			'style' => $style,
			'class' => 'defaultfont'
		]);
		$clSelect->addOption("#none#", g_l('wysiwyg', '[none]'));
		$classesCSV = empty($this->args['cssclasses']) ?: trim($this->args['cssclasses'], ",");
		if($classesCSV){
			foreach(explode(',', $classesCSV) as $val){
				$clSelect->addOption($val, "." . $val);
			}
		}
		if(!empty($this->args['cssclass'])){
			$clSelect->selectOption($this->args['cssclass']);
		}

		return $clSelect->getHTML() . we_html_element::hidden('we_dialog_args[cssclasses]', $classesCSV);
	}

}
