<?php
/**
 * webEdition CMS
 *
 * $Rev: cfcecc6a1959 $
 * $Author: Marc Krämer <marc.kraemer@aldente-it.de> $
 * $Date: 2019-01-17 10:20 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * class    listviewBase
 * @desc    This is the base class for all webEdition listviews.
 *
 */
abstract class we_listview_base{
	const PROPPREFIX = 'we_';
	const CATEGORY_FIELD = self::PROPPREFIX . 'category';
	const CLASS_FIELD = self::PROPPREFIX . 'class_id';
	const CONTENTTYPE_FIELD = self::PROPPREFIX . 'contenttype';
	const CUSTOMER_FIELD = self::PROPPREFIX . 'webuserid';
	const CUSTOMER_PREFIX = self::PROPPREFIX . 'customer_';
	const DATECREATION_FIELD = self::PROPPREFIX . 'creationdate';
	const DATEMOD_FIELD = self::PROPPREFIX . 'moddate';
	const DATEPUBLISHED_FIELD = self::PROPPREFIX . 'published';
	const DESC_FIELD = self::PROPPREFIX . 'description';
	const DOCTYPE_FIELD = self::PROPPREFIX . 'doctype';
	const FILENAME_FIELD = self::PROPPREFIX . 'filename';
	const HREF_FIELD = self::PROPPREFIX . 'href';
	const ID_FIELD = self::PROPPREFIX . 'id';
	const KEYWORDS_FIELD = self::PROPPREFIX . 'keywords';
	const LANG_FIELD = self::PROPPREFIX . 'language';
	const LASTPATH_FIELD = self::PROPPREFIX . 'lastpath';
	const OBJECT_FIELD_PREFIX = self::PROPPREFIX . 'object_';
	const PARENTID_FIELD = self::PROPPREFIX . 'parentid';
	const PATH_FIELD = self::PROPPREFIX . 'path';
	const RANDOM_FIELD = 'random()';
	const SHOPVARIANTS_FIELD = self::PROPPREFIX . 'shopvariants';
	const TEXT_FIELD = self::PROPPREFIX . 'text';
	const TITLE_FIELD = self::PROPPREFIX . 'title';
	const TRIGGER_FIELD = self::PROPPREFIX . 'triggerid';
	const URL_FIELD = self::PROPPREFIX . 'url';
	const VARIANT_FIELD = self::PROPPREFIX . 'variant';
	const WEEKDAYS = ['sunday', 'monday', 'tuesday', 'wednesday', 'thursday', 'friday', 'saturday'];

	protected $DB_WE; /* Main DB Object */
	var $name; /* name of listview */
	protected $rows = -1; /* Number of rows */
	protected $cols = 0; /* Number of cols */
	protected $maxItemsPerPage = 1;
	protected $stop_next_row = false;
	var $start = 0; /* Where to start output */
	protected $search = ''; /* search words */
	protected $offset = 0; /* start offset of first page */
	protected $order = ''; /* Order string */
	protected $desc = false; /* set to true, if order should be descendend */
	var $cats = ''; /* category string */
	var $catOr = false; /* set to true if it should be an 'OR condition' e.g. categories='value1' OR categories='value2' */
	protected $anz_all = 0; /* total number of matches */
	var $anz = 0; /* number of rows in page */
	var $workspaceID = ''; /* commaseperated string of id's of workspace */
	var $count = 0; /* internal counter */
	protected $Record = []; /* array to store results */
	private $close_a = true; /* close </a> when endtag used */
	protected $customerFilterType = false; // shall we control customer-filter?
	public $calendar_struct = [];
	var $id = 0;
	protected $IDs = []; /* array of ids with pages which are found */
	protected $condition = '';
	public $hidedirindex = false; //since $lv->hidedirindex is accessed at output
	protected $searchable = true;
	protected $customers = '';
	protected static $lvNr = 0;

	/**
	 * listviewBase()
	 * @desc    constructor of class
	 *
	 * @param   name         string - name of listview
	 * @param   rows         integer - number of rows to display per page
	 * @param   offset       integer - start offset of first page
	 * @param   order        string - field name(s) to order by
	 * @param   desc         boolean - set to true, if order should be descendend
	 * @param   cats         string - comma separated categories
	 * @param   catOr        boolean - set to true if it should be an "OR condition"
	 * @param   workspaceID  string - commaseperated string of id's of workspace
	 * @param   cols   		  integer - to display a table this is the number of cols
	 *
	 */
	function __construct($name = 0, $rows = 999999999, $offset = 0, $order = '', $desc = false, $cats = '', $catOr = false, array $workspaceID = [], $cols = 0, $calendar = '', $datefield = '', $date = '', $weekstart = '', $categoryids = '', $customerFilterType = 'all', $id = 0){
		$this->rows = $rows;
		$this->order = $order;
		$this->desc = $desc;
		$this->cats = trim($cats);
		$this->categoryids = trim($categoryids);
		$this->catOr = $catOr;
		$this->workspaceID = $workspaceID ?: [];
		$this->customerFilterType = $customerFilterType;
		$this->id = $id;
		$this->offset = max(0, $offset);

		self::$lvNr++;
		$this->name = $name ?: (self::$lvNr);
		//? strange setting - comes from we_tag_search
		$this->search = trim(str_replace(['"', '\\"'], '', (!($val = we_base_request::_(we_base_request::STRING, 'we_lv_search_' . $this->name, '')) && we_base_request::_(we_base_request::BOOL, 'we_from_search_' . $this->name)) ? -1 : $val));
		$this->DB_WE = new DB_WE();
		$this->maxItemsPerPage = $cols ? ($rows * $cols) : $rows;
		$this->cols = (($cols == '' && ($calendar === 'month' || $calendar === 'month_table')) ? 7 : $cols);
		$this->start = max(we_base_request::_(we_base_request::INT, 'we_lv_start_' . $this->name, $this->offset), $this->offset);

		if($calendar){
			$this->calendar_struct = [
				'calendar' => $calendar,
				'defaultDate' => ($calendar ? ($date ? strtotime($date) : time()) : ''),
				'date' => -1,
				'calendarCount' => '',
				'datefield' => ($calendar ? ($datefield ?: '###Published###') : ''),
				'start_date' => '',
				'end_date' => '',
				'storage' => [],
				'forceFetch' => false,
				'count' => 0,
				'weekstart' => ($weekstart && ($match = array_search($weekstart, self::WEEKDAYS)) !== false ? $match : 0),
			];
		}
	}

	protected function getIdQuery($fieldname){
		return ($this->id ? $fieldname . ' IN(' . $this->id . ') ' : '');
	}

	/**
	 * next_record()
	 * @desc    selects next row and returns true. if no more row exists, it returns false
	 *
	 *
	 */
	public function next_record(){
		// overwrite
		if(!$this->calendar_struct){
			return false;
		}
		$this->calendar_struct['calendarCount'] ++;
		$this->calendar_struct['count'] = $this->count;
		$this->calendar_struct['forceFetch'] = false;
		$calendarCount = $this->calendar_struct['calendarCount'];
		if(($calendarCount > 0 || ($this->calendar_struct['calendar'] === 'day' && $calendarCount >= 0)) && $calendarCount <= $this->calendar_struct['numofentries']){
			if($this->calendar_struct['date'] < 0){
				$this->calendar_struct['date'] = $this->calendar_struct['defaultDate'];
			}
			$date = $this->calendar_struct['date'];
			$month = date('m', $date);
			$year = date('Y', $date);
			$day = date('j', $date);
			switch($this->calendar_struct['calendar']){
				case 'year':
					$date = mktime(0, 0, 0, $calendarCount, $day, $year);
					$start_date = mktime(0, 0, 0, $calendarCount, $day, $year);
					$end_date = mktime(23, 59, 59, $calendarCount, self::getNumberOfDays($calendarCount, $year), $year);
					break;
				case 'day':
					$date = mktime($calendarCount, 0, 0, $month, $day, $year);
					$start_date = mktime($calendarCount, 0, 0, $month, $day, $year);
					$end_date = mktime($calendarCount, 59, 59, $month, $day, $year);
					break;
				default:
					$date = mktime(0, 0, 0, $month, $calendarCount, $year);
					$start_date = mktime(0, 0, 0, $month, $calendarCount, $year);
					$end_date = mktime(23, 59, 59, $month, $calendarCount, $year);
			}

			$this->calendar_struct['date'] = $date;
			$this->calendar_struct['date_human'] = date('Y-m-d', $date);
			$this->calendar_struct['day_human'] = $day;
			$this->calendar_struct['month_human'] = $month;
			$this->calendar_struct['year_human'] = $year;
			$this->calendar_struct['start_date'] = $start_date;
			$this->calendar_struct['end_date'] = $end_date;

			foreach($this->calendar_struct['storage'] as $k => $v){
				if($v >= $start_date && $v <= $end_date){
					$found = array_search($k, $this->IDs, false);
					if($found !== false){
						$this->calendar_struct['forceFetch'] = true;
						$this->calendar_struct['count'] = $found;
					}
				}
			}
		}
		if($calendarCount > $this->calendar_struct['numofentries']){
			$this->calendar_struct['date'] = 0;
		}
		if(!$this->calendar_struct['forceFetch']){
			$this->count++;
		}
		$this->Record = [];
	}

	/**
	 * f()
	 * @desc    returns content of a field
	 *
	 * @param   key  string - name of field to return
	 *
	 */
	function f($key, $stripWE = true){
		$key = strtolower(preg_replace('/^(of|wedoc|we)_/i', ($stripWE ? '' : self::PROPPREFIX), $key));
		return (isset($this->Record[$key]) ? $this->Record[$key] : '');
	}

	/**
	 * hasNextPage()
	 * @desc    returns true if next page exists, otherwise it returns false
	 *
	 * @param   name  string - name of listview
	 *
	 */
	public function hasNextPage($parentEnd = false){
		if(!empty($this->calendar_struct['calendar'])){
			return true;
		}
		if($parentEnd && ($end = we_base_request::_(we_base_request::INT, 'we_lv_pend_' . $this->name))){
			return (($this->start + $this->anz) < $end);
		}
		return (($this->start + $this->anz) < $this->anz_all);
	}

	/**
	 * hasPrevPage()
	 * @desc    returns true if previous page exists, otherwise it returns false
	 *
	 * @param   name  string - name of listview
	 *
	 */
	public function hasPrevPage($parentStart = false){
		if(!empty($this->calendar_struct['calendar'])){
			return true;
		}
		if($parentStart && ($start = we_base_request::_(we_base_request::INT, 'we_lv_pstart_' . $this->name, 0))){
			return (abs($this->start) != $start);
		}
		return (abs($this->start) > abs($this->offset));
	}

	/**
	 * getBackLink()
	 * @desc    gets the HTML Code for the back link
	 *
	 * @param   attribs  array - array with HTML attributes
	 *
	 */
	public function getBackLink($attribs, $content = ''){
		$only = we_weTag_tag::getAttribute('only', $attribs, '', we_base_request::STRING);
		$urlID = we_weTag_tag::getAttribute('id', $attribs, 0, we_base_request::INT);
		if(!empty($this->calendar_struct['calendar'])){

			$month = $this->calendar_struct['month_human'];
			$day = $this->calendar_struct['day_human'];
			$year = $this->calendar_struct['year_human'];
			switch($this->calendar_struct['calendar']){
				case 'month':
				case 'month_table':
					$day = 1;
					$month--;
					if($month < 1){
						$month = 12;
						$year--;
					}
					break;
				case 'year':
					$year--;
					break;
				case 'day':
					$day--;
					if($day < 1){
						$month--;
						if($month <= 1){
							$month = 12;
							$year--;
						}
						$day = self::getNumberOfDays($month, $year);
					}
					break;
			}
			$newdate = $year . '-' . $month . '-' . $day;

			$href = we_tag('url', ['id' => ($urlID ?: 'top'), 'hidedirindex' => $this->hidedirindex]);
			$attribs['href'] = $href . (strpos($href, '?') === false ? '?' : '&');
			$attribs['rel'] = 'prev';

			$tmp_href = self::we_makeQueryString(['we_lv_calendar_' . $this->name => $this->calendar_struct['calendar'], 'we_lv_datefield_' . $this->name => $this->calendar_struct['datefield'],
					'we_lv_date_' . $this->name => $newdate]);
		} else if(!$this->hasPrevPage()){
			return $content;
		}
		$foo = max($this->offset, $this->start - $this->maxItemsPerPage);
		$tmp_href = self::we_makeQueryString(['we_lv_start_' . $this->name => $foo]);

		$href = we_tag('url', ['id' => ($urlID ?: 'top'), 'hidedirindex' => $this->hidedirindex]);
		$attribs['href'] = $href . (strpos($href, '?') === false ? '?' : '&') . $tmp_href;
		if($only){
			$this->close_a = false;
			return (isset($attribs[$only]) ? $attribs[$only] : '');
		}
		return getHtmlTag('a', $attribs, $content, true);
	}

	static function we_makeQueryString(array $query = [], $filter = []){
		//remove potential Cookies and filter from query
		$filterArr = array_merge([//filter special variables
			'edit_object',
			'edit_document',
			'we_editObject_ID',
			'we_editDocument_ID',
			'we_transaction',
			'we_cmd',
			'we_cmd[1]',
			'pv_id',
			'pv_tid',
			'jwt',
			'bsuniquevid',
			's'//password-form
			], array_keys($query), array_keys($filter), array_keys($_COOKIE), (TAGLINKS_OBJECTSEOURLS && !empty($GLOBALS['we']['main']['doc']->Url) && show_SeoLinks() ?
			[WEOBJ_PARAM, 'we_oid'] : [])
		);

		if(!empty($_REQUEST)){
			foreach($_REQUEST as $key => $val){
				if((!in_array($key, $filterArr)) && (strpos($key, 'we_ui_') !== 0)){
					$query[$key] = $val;
				}
			}
		}
		return http_build_query($query);
	}

	/**
	 * getNextLink()
	 * @desc    gets the HTML Code for the next link
	 *
	 * @param   attribs  array - array with HTML attributes
	 *
	 */
	public function getNextLink($attribs, $content = ''){
		$only = we_weTag_tag::getAttribute('only', $attribs, '', we_base_request::STRING);
		$urlID = we_weTag_tag::getAttribute('id', $attribs, 0, we_base_request::INT);
		if(!empty($this->calendar_struct['calendar'])){

			$month = $this->calendar_struct['month_human'];
			$day = $this->calendar_struct['day_human'];
			$year = $this->calendar_struct['year_human'];
			switch($this->calendar_struct['calendar']){
				case 'month':
				case 'month_table':
					$day = 1;
					$month++;
					if($month > 12){
						$month = 1;
						$year++;
					}
					break;
				case 'year':
					$year++;
					break;
				case 'day':
					$day++;
					$numd = self::getNumberOfDays($month, $year);
					if($day > $numd){
						$day = 1;
						$month++;
					}
					if($month >= 12){
						$month = 1;
						$year++;
					}
					break;
			}
			$newdate = $year . '-' . $month . '-' . $day;
			$tmp_href = self::we_makeQueryString(['we_lv_calendar_' . $this->name => $this->calendar_struct['calendar'], 'we_lv_datefield_' . $this->name => $this->calendar_struct['datefield'],
					'we_lv_date_' . $this->name => $newdate]);
		} else if(!$this->hasNextPage()){
			return $content;
		} else {
			$foo = $this->start + $this->maxItemsPerPage;
			$tmp_href = self::we_makeQueryString(['we_lv_start_' . $this->name => $foo]);
		}

		$href = we_tag('url', ['id' => ($urlID ?: 'top'), 'hidedirindex' => $this->hidedirindex]);
		$attribs['href'] = $href . (strpos($href, '?') === false ? '?' : '&') . $tmp_href;
		$attribs['rel'] = 'next';
		if($only){
			$this->close_a = false;
			return (isset($attribs[$only]) ? $attribs[$only] : '');
		}
		return getHtmlTag('a', $attribs, $content, true);
	}

	public function shouldPrintEndTR(){
		return ($this->cols && ($this->count % $this->cols) === 0);
	}

	public function shouldPrintStartTR(){
		return ($this->cols && (($this->count - 1) % $this->cols) === 0);
	}

	public function tdEmpty(){
		return ($this->count > $this->anz);
	}

	public function getCalendarField($type, $langcode){
		$showDate = ($this->calendar_struct['date'] > 0 ? $this->calendar_struct['date'] : $this->calendar_struct['defaultDate']);
		switch($type){
			case 'day':
				return ($this->calendar_struct['calendar'] === 'day' ?
					date('j', $this->calendar_struct['defaultDate']) :
					($this->calendar_struct['date'] ? date('j', $this->calendar_struct['date']) : '')
					);
			case 'dayname':
			case 'dayname_long':
				$days = we_base_country::getTranslationList(we_base_country::DAY, $langcode);
				$day = strtolower(date('D', $showDate));
				return $days['wide'][$day];
			case 'dayname_short':
				$days = we_base_country::getTranslationList(we_base_country::DAY, $langcode);
				$day = strtolower(date('D', $showDate));
				return $days['abbreviated'][$day];
			case 'month':
				return date('m', ($this->calendar_struct['date'] ?: $this->calendar_struct['defaultDate']));
			case 'monthname':
			case 'monthname_long':
				$months = we_base_country::getTranslationList(we_base_country::MONTH, $langcode);
				$mon = date('n', $showDate);
				return $months['wide'][$mon];
			case 'monthname_short':
				$months = we_base_country::getTranslationList(we_base_country::MONTH, $langcode);
				$mon = date('n', $showDate);
				return $months['abbreviated'][$mon];
			case 'year':
				return date('Y', $showDate);
			case 'hour':
				return date('H:i', $showDate);
			case 'week':
				return date('W', $showDate);
			default:
				return ($this->calendar_struct['date'] ? date('j', $this->calendar_struct['date']) : '');
		}
	}

	public function getCalendarFieldValue($name, $langcode){
		$showDate = ($this->calendar_struct['date'] ?: $this->calendar_struct['defaultDate']);
		switch($name){
			case 'day':
				return date('j', $showDate);
			case 'month':
				return date('m', $showDate);
			case 'year':
				return date('Y', $showDate);
			case 'dayname':
			case 'dayname_long':
				$days = we_base_country::getTranslationList(we_base_country::DAY, $langcode);
				$day = strtolower(date('D', $showDate));
				return $days['wide'][$day];
			case 'dayname_short':
				$days = we_base_country::getTranslationList(we_base_country::DAY, $langcode);
				$day = strtolower(date('D', $showDate));
				return $days['abbreviated'][$day];
			case 'monthname':
			case 'monthname_long':
				$months = we_base_country::getTranslationList(we_base_country::MONTH, $langcode);
				$mon = date('n', $showDate);
				return $months['wide'][$mon];
			case 'monthname_short':
				$months = we_base_country::getTranslationList(we_base_country::MONTH, $langcode);
				$mon = date('n', $showDate);
				return $months['abbreviated'][$mon];
			case 'hour':
				return date('H:i', $showDate);
			case 'start_date':
				return $this->calendar_struct['start_date'];
			case 'end_date':
				return $this->calendar_struct['end_date'];
			case 'timestamp':
				return $showDate;
			default:
				return date('j', $showDate);
		}
	}

	public function isCalendarField($type){
		return in_array($type, ['day', 'dayname', 'dayname_long', 'dayname_short', 'month', 'monthname', 'monthname_long', 'monthname_short', 'year', 'hour']);
	}

	protected function fetchCalendar(&$condition, &$calendar_select, &$calendar_where, $matrix = []){
		if(!$this->calendar_struct['calendar']){
			return;
		}
		$calendar = $this->calendar_struct['calendar'];
		$day = date('j', $this->calendar_struct['defaultDate']);
		$month = date('m', $this->calendar_struct['defaultDate']);
		$year = date('Y', $this->calendar_struct['defaultDate']);

		switch($calendar){
			case 'year':
				$start_date = mktime(0, 0, 0, 1, $day, $year);
				$end_date = mktime(23, 59, 59, 12, $day, $year);
				$numofentries = 12;
				break;
			case 'day':
				$start_date = mktime(0, 0, 0, $month, $day, $year);
				$end_date = mktime(23, 59, 59, $month, $day, $year);
				$numofentries = 24;
				break;
			default:
				$numofentries = self::getNumberOfDays($month, $year);
				$start_date = mktime(0, 0, 0, $month, 1, $year);
				$end_date = mktime(23, 59, 59, $month, $numofentries, $year);
		}

		$this->calendar_struct['date_human'] = date('Y-m-d', $this->calendar_struct['defaultDate']);
		$this->calendar_struct['day_human'] = $day;
		$this->calendar_struct['month_human'] = $month;
		$this->calendar_struct['year_human'] = $year;
		$this->calendar_struct['numofentries'] = $numofentries;
		$this->calendar_struct['start_date'] = $start_date;
		$this->calendar_struct['end_date'] = $end_date;


		if(!$this->calendar_struct['datefield'] || $this->calendar_struct['datefield'] === '###Published###'){
			$this->calendar_struct['datefield'] = '###Published###';
			$calendar_select = ',f.Published AS Calendar ';
			$calendar_where = '(f.Published>=' . $start_date . ' AND f.Published<=' . $end_date . ') ';
		} else {
			$field = ($matrix && in_array($this->calendar_struct['datefield'], array_keys($matrix))) ?
				$matrix[$this->calendar_struct['datefield']]['alias'] . '.' . $matrix[$this->calendar_struct['datefield']]['type'] . '_' . $this->calendar_struct['datefield'] :
				CONTENT_TABLE . '.Dat';

			$calendar_select = ',' . $field . ' AS Calendar ';
			$condition = ($condition ? $condition . ' AND ' : '') . $this->calendar_struct['datefield'] . '>=' . $start_date . ' AND ' . $this->calendar_struct['datefield'] . '<=' . $end_date;
		}
	}

	function postFetchCalendar(){
		if(!$this->calendar_struct['calendar']){
			return;
		}
		$start = 0;
		$this->anz = $this->calendar_struct['numofentries'] + $start;
		$this->anz_all = $this->anz;

		switch($this->calendar_struct['calendar']){
			case 'day':
				$this->calendar_struct['calendarCount'] = -1;
				break;
			case 'month':
			case 'month_table':
				$start = (int) date('w', strtotime(date('Y', $this->calendar_struct['defaultDate']) . '-' . date('m', $this->calendar_struct['defaultDate']) . '-1'));
				if($this->calendar_struct['weekstart'] != ''){
					$start = $start - $this->calendar_struct['weekstart'];
					if($start < 0){
						$start = 7 + $start;
					}
				}
				$this->anz += start;
				$this->anz_all = $this->anz;
				$this->calendar_struct['calendarCount'] = 0 - $start;
				$rest = $this->cols - ($this->anz % $this->cols);
				if($rest < $this->cols){
					$this->anz += $rest;
					$this->anz_all += $rest;
				}
				break;
			default:
				$this->calendar_struct['calendarCount'] = 0;
		}

		$this->calendar_struct['date'] = -1;
	}

	/**
	 * Returns number od days for given month
	 *
	 * @param          int                                  $month
	 * @param          int                                  $year
	 *
	 *
	 * @return         int
	 */
	static function getNumberOfDays($month, $year){
		switch($month){
			case 1:
			case 3:
			case 5:
			case 7:
			case 8:
			case 10:
			case 12:
				return 31;
			case 2:
				return ($year % 4) == 0 ? 29 : 28;
			default:
				return 30;
		}
	}

	/**
	 * @return boolean
	 * @desc returns, if tag we:next / we:back should set an endtag automatically. As default
	 *       it should set one
	 */
	function close_a(){
		$close = $this->close_a;
		$this->close_a = true;
		return $close;
	}

	/**
	 * @deprecated
	 * @return array
	 */
	public function getDBRecord(){
		return $this->DB_WE->getRecord();
	}

	public function getRecord(){
		return $this->Record;
	}

	public function getCustomerRestrictionQuery($specificCustomersQuery, $classID, $mfilter, $listQuery){
		return '';
	}

	public function getFoundDocument(){
		static $doc = null;
		static $id = 0;
		if($id == ($docID = $this->f(self::ID_FIELD))){
			return $doc;
		}
		$id = $docID;
		$model = new we_webEditionDocument();
		$model->initByID($docID);
		$doc = $model;
		return $doc;
	}

	public function getListviewRequest($lvname){
		return array_merge(array_filter([
			'we_lv_ct_' . $lvname => $this->contentTypes,
			'we_lv_order_' . $lvname => $this->order,
			'we_lv_desc_' . $lvname => $this->desc,
			'we_lv_cats_' . $lvname => $this->cats,
			'we_lv_catOr_' . $lvname => $this->catOr,
			'we_lv_ws_' . $lvname => implode(',', $this->workspaceID),
			'we_lv_condition_' . $lvname => $this->condition,
			]), ((isset($this->searchable) && !$this->searchable) ? ['we_lv_se_' . $lvname => 0] : [])
		);
	}

	public static function preFixArrayKeys(array $array){
		return array_combine(
			array_map(function($k){
				return self::PROPPREFIX . $k;
			}, array_keys($array)), $array);
	}

	public function getDID(){
		return $this->IDs[$this->count - 1];
	}

	public function getCalendarStruct(){
		return $this->calendar_struct;
	}

	public function getMaxPages(){
		return $this->rows ? ceil(((float) $this->anz_all - abs($this->offset)) / ((float) $this->maxItemsPerPage )) : 1;
	}

	public function getPageNo(){
		return $this->rows ? (((abs($this->start) - abs($this->offset)) / $this->maxItemsPerPage) + 1) : 1;
	}

	public function getStart(){
		return $this->start + 1 - abs($this->offset);
	}

	public function getEnd(){
		return $this->rows ? min(($this->start - abs($this->offset)) + ($this->maxItemsPerPage), ($this->anz_all - abs($this->offset))) : ($this->anz_all - abs($this->offset));
	}

	public function getRows(){
		return $this->anz_all - abs($this->offset);
	}

	public function getMaxItemsPerPage(){
		return $this->maxItemsPerPage;
	}

}
