<?php

/**
 * webEdition CMS
 *
 * $Rev$
 * $Author$
 * $Date$
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_navigation_ruleFrames{
	public $Frameset;
	public $Controller;
	public $db;
	public $jsCmd = null;

	function __construct(){
		$this->Frameset = WEBEDITION_DIR . 'we_showMod.php?mod=navigation';
		$this->Controller = new we_navigation_ruleControl();
		$this->db = new DB_WE();
		$weSuggest = &we_gui_suggest::getInstance();
		$this->jsCmd = new we_base_jsCmd();
	}

	function getHTML($what){
		switch($what){
			case 'ruleFrameset':
				echo $this->getHTMLFrameset();
				break;
			case 'ruleContent':
				echo $this->getHTMLContent();
				break;
			default:
				t_e(__FILE__ . ": unknown reference $what");
		}
	}

	function getHTMLFrameset(){
		return we_html_tools::getHtmlTop(g_l('navigation', '[menu_highlight_rules]'), '', '', '', we_html_element::body(['class ' => 'weDialogBody']
					, we_html_element::iframe('content', WEBEDITION_DIR . 'we_showMod.php?mod=navigation&pnt=ruleContent', 'position:absolute;top:0;bottom:0;left:0;right:0;overflow: hidden') .
					we_html_element::iframe('cmdFrame', "about:blank", 'position:absolute;bottom:0;height:0;left:0;right:0;overflow: hidden;')
		));
	}

	function getHTMLContent(){
		// content contains textarea with all so far existing rules
		$weSuggest = & we_gui_suggest::getInstance();

		$allRules = we_navigation_ruleControl::getAllNavigationRules();

		$rules = [];

		foreach($allRules as $navigationRule){
			$rules[$navigationRule->ID] = $navigationRule->NavigationName;
		}
		asort($rules);

		$parts = [
			[
				'headline' => g_l('navigation', '[rules][available_rules]'),
				'space' => we_html_multiIconBox::SPACE_BIG,
				'html' => '<table class="default">
	<tr><td>' . we_html_tools::select('navigationRules', $rules, 8, '', false, ['style' => "width: 275px;", 'onclick' => 'we_cmd(\'navigation_edit_rule\', this.value)']) . '</td>
		<td style="vertical-align:top">' . we_html_button::create_button(we_html_button::NEW_ENTRY, 'javascript:we_cmd("new_navigation_rule")') . '<div style="height:10px;"></div>' . we_html_button::create_button(we_html_button::DELETE, 'javascript:we_cmd("delete_navigation_rule")') . '
		</td>
	</tr>
</table>'
			], [
				'headline' => g_l('navigation', '[rules][rule_name]'),
				'space' => we_html_multiIconBox::SPACE_BIG,
				'html' => we_html_tools::textInput('NavigationName', '', '', ['style' => "width: 275px;"]),
				'noline' => true
			],
		];

		$weSuggest->setAcId('NavigationIDPath');
		$weSuggest->setContentType([we_base_ContentTypes::FOLDER, we_base_ContentTypes::NAVIGATION]);
		$weSuggest->setInput('NavigationIDPath');
		$weSuggest->setMaxResults(10);
		$weSuggest->setTable(NAVIGATION_TABLE);
		$weSuggest->setResult('NavigationID');
		$weSuggest->setSelector(we_gui_suggest::DocSelector);
		$weSuggest->setWidth(275);
		$weSuggest->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_file', document.we_form.elements.NavigationID.value,WE().consts.tables.NAVIGATION_TABLE, 'NavigationID', 'NavigationIDPath')"));


		$parts[] = [
			'headline' => g_l('navigation', '[rules][rule_navigation_link]'),
			'space' => we_html_multiIconBox::SPACE_BIG,
			'html' => $weSuggest->getHTML(),
			'noline' => true
		];

		$selectionTypes = [we_navigation_navigation::DYN_DOCTYPE => g_l('global', '[documents]')];
		if(we_modules_info::isActive(we_modules_info::OBJECT)){
			$selectionTypes[we_navigation_navigation::DYN_CLASS] = g_l('global', '[objects]');
		}

		$parts[] = [
			'headline' => g_l('navigation', '[rules][rule_applies_for]'),
			'space' => we_html_multiIconBox::SPACE_BIG,
			'html' => we_html_tools::select('SelectionType', $selectionTypes, 1, 0, false, ['style' => "width: 275px;", 'onchange' => "switchType(this.value);"])
		];

// getDoctypes
		$docTypes = [0 => g_l('navigation', '[no_entry]')];
		$dtq = we_doctype_docTypes::getDoctypeQuery($this->db);
		$this->db->query('SELECT dt.ID,dt.DocType FROM ' . DOC_TYPES_TABLE . ' dt LEFT JOIN ' . FILE_TABLE . ' dtf ON dt.ParentFileID=dtf.ID ' . $dtq['join'] . ' WHERE ' . $dtq['where']);
		while($this->db->next_record()){
			$docTypes[$this->db->f('ID')] = $this->db->f('DocType');
		}

		$weSuggest->setAcId("FolderIDPath");
		$weSuggest->setContentType(we_base_ContentTypes::FOLDER);
		$weSuggest->setInput('FolderIDPath');
		$weSuggest->setMaxResults(10);
		$weSuggest->setResult('FolderID');
		$weSuggest->setSelector(we_gui_suggest::DirSelector);
		$weSuggest->setWidth(275);
		$weSuggest->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_directory', document.we_form.elements.FolderID.value,WE().consts.tables.FILE_TABLE, 'FolderID', 'FolderIDPath')"));
		$weSuggest->setTrashButton(we_html_button::create_button(we_html_button::TRASH, "javascript:document.we_form.elements.FolderID.value = '';document.we_form.elements.FolderIDPath.value = '';"));


		$parts[] = ['headline' => g_l('navigation', '[rules][rule_folder]'),
			'html' => $weSuggest->getHTML(),
			'space' => we_html_multiIconBox::SPACE_ICON,
			'noline' => true,
			'rightHeadline' => true,
			'icon' => we_html_multiIconBox::PROP_PATH,
			'class' => ' switch switch_doctype'
		];
		$parts[] = ['headline' => g_l('navigation', '[rules][rule_doctype]'),
			'html' => we_html_tools::select('DoctypeID', $docTypes, 1, 0, false, ['style' => "width: 275px;"]),
			'space' => we_html_multiIconBox::SPACE_ICON,
			'noline' => true,
			'rightHeadline' => true,
			'class' => ' switch switch_doctype'
		];

		if(we_modules_info::isActive(we_modules_info::OBJECT)){
			$weSuggest->setAcId("ClassIDPath");
			$weSuggest->setContentType([we_base_ContentTypes::FOLDER, we_base_ContentTypes::OBJECT]);
			$weSuggest->setInput("ClassIDPath");
			$weSuggest->setMaxResults(10);
			$weSuggest->setResult('ClassID');
			$weSuggest->setSelector(we_gui_suggest::DocSelector);
			$weSuggest->setTable(OBJECT_TABLE);
			$weSuggest->setWidth(275);
			$weSuggest->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',document.we_form.elements.ClassID.value,WE().consts.tables.OBJECT_TABLE,'ClassID','ClassIDPath','get_workspaces')"));

			$weAcSelector = $weSuggest->getHTML();
			$parts[] = ['headline' => g_l('navigation', '[rules][rule_class]'),
				'html' => $weAcSelector,
				'space' => we_html_multiIconBox::SPACE_ICON,
				'noline' => true,
				'rightHeadline' => true,
				'icon' => we_html_multiIconBox::PROP_CLASS,
				'class' => ' switch switch_classname'
			];
			$parts[] = ['headline' => g_l('navigation', '[rules][rule_workspace]'),
				'html' => we_html_tools::select('WorkspaceID', [], 1, '', false, ['style' => "width: 275px;"]),
				'space' => we_html_multiIconBox::SPACE_ICON,
				'noline' => true,
				'rightHeadline' => true,
				'class' => ' switch switch_classname'
			];
		}

		$parts[] = ['headline' => g_l('navigation', '[rules][rule_categories]'),
			'html' => $this->getHTMLCategory(),
			'space' => we_html_multiIconBox::SPACE_ICON,
			'preline' => true,
			'rightHeadline' => true,
			'fold' => true,
			'hide' => !we_modules_info::isActive(we_modules_info::CATEGORY),
			'icon' => we_html_multiIconBox::PROP_CATEGORIES
		];

		$saveButton = we_html_button::create_button(we_html_button::SAVE, 'javascript:we_cmd("save_navigation_rule");');
		$closeButton = we_html_button::create_button(we_html_button::CLOSE, 'javascript:top.window.close();');
		return we_html_tools::getHtmlTop('', '', '', we_html_element::jsScript(JS_DIR . 'multiIconBox.js') .
				we_html_element::jsScript(JS_DIR . 'formFunctions.js') .
				we_html_element::jsScript(WE_JS_MODULES_DIR . 'navigation/navigationRule.js', '', ['id' => 'loadVarNavigationRules', 'data-rules' => setDynamicVar([
						'dependencies' => [
							we_navigation_navigation::DYN_CLASS => ["ClassID", "WorkspaceID", "Categories"],
							we_navigation_navigation::DYN_DOCTYPE => ["FolderID", "DoctypeID", "Categories"]
						],
						'trashButton' => we_html_button::create_button(we_html_button::TRASH, 'javascript:' . we_base_constants::PLACEHOLDER)
				])]), we_html_element::body(['onload' => "switchType(document.we_form.SelectionType.value)",
					'class' => 'weDialogBody'], we_html_element::form(['name' => 'we_form', 'target' => "cmdFrame", 'method' => "post", 'action' => WEBEDITION_DIR . 'we_showMod.php?mod=navigation&pnt=ruleCmd'], we_html_element::hiddens([
							'cmd' => '',
							'ID' => '0'
						]) .
						we_html_multiIconBox::getHTML('navigationRules', $parts, true, we_html_button::position_yes_no_cancel($saveButton, null, $closeButton), -1, '', '', false, g_l('navigation', '[rules][navigation_rules]')))
		));
	}

	private function getHTMLCategory(){
		$addbut = we_html_button::create_button(we_html_button::ADD, "javascript:we_cmd('we_selector_document',0,WE().consts.tables.CATEGORY_TABLE,'','','addCat')");

		$table = new we_html_table(['id' => 'CategoriesBlock', 'style' => 'display: block;', 'class' => 'default withSpace'], 2, 1);

		$table->setColContent(0, 0, we_html_element::div([
				'id' => 'categories',
				'class' => 'blockWrapper',
				'style' => 'width: 380px; height: 80px; border: #AAAAAA solid 1px;'
		]));

		$table->setCol(1, 0, ['colspan' => 2, 'style' => 'text-align:right'], we_html_button::create_button(we_html_button::DELETE_ALL, "javascript:removeAllCats()") . $addbut);

		return $table->getHtml() .
			we_html_element::hiddens([
				'CategoriesControl' => 0,
		]);
	}

	public function process(we_base_jsCmd $jscmd){
		$this->Controller->processVariables();
		$this->Controller->processCommands($jscmd);
	}

}
