<?php
/**
 * webEdition CMS
 *
 * $Rev$
 * $Author$
 * $Date$
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * Definition of WebEdition Newsletter Base
 *
 */
class we_newsletter_base{
	const STATUS_ERROR = -1;
	const STATUS_SUCCESS = 0;
	const STATUS_EMAIL_EXISTS = 1;
	const STATUS_EMAIL_INVALID = 2;
	const STATUS_CONFIRM_FAILED = 3;
	const FEMALE_SALUTATION_FIELD = 'female_salutation';
	const MALE_SALUTATION_FIELD = 'male_salutation';
	const EMAIL_REPLACE_TEXT = '###EMAIL###';
	const FIRSTNAME_REPLACE_TEXT = '###FIRSTNAME###';
	const LASTNAME_REPLACE_TEXT = '###LASTNAME###';
	const CUSTOMERID_REPLACE_TEXT = '###CUSTOMERID###';
	const TITLE_REPLACE_TEXT = '###TITLE###';

	var $db;
	var $table;
	var $persistents = [];

	/**
	 * Default Constructor
	 */
	function __construct(){
		$this->db = new DB_WE();
	}

	/**
	 * Load entry from database
	 *
	 * @param Int $id
	 * @return Boolean
	 */
	function load($id = 0){
		if($id){
			$this->ID = intval($id);
		}
		if(!$this->ID){
			return false;
		}

		if(($hash = getHash('SELECT * FROM ' . $this->db->escape($this->table) . ' WHERE ID=' . intval($this->ID), $this->db))){
			foreach($hash as $key => $value){
				if(isset($this->persistents[$key])){
					$this->{$key} = $value;
				}
			}
		}

		return true;
	}

	/**
	 * save entry in database
	 */
	function save(){
		$sets = $wheres = [];
		foreach(array_keys($this->persistents) as $val){
			$sets[$val] = $this->$val;
		}

		unset($sets['ID']);
		$set = we_database_base::arraySetter($sets);

		if($this->ID){
			return $this->db->query('UPDATE ' . $this->db->escape($this->table) . ' SET ' . $set . ' WHERE ID=' . intval($this->ID));
		}
		$ret = $this->db->query('INSERT INTO ' . $this->db->escape($this->table) . ' SET ' . $set);
		# get ID #
		$this->ID = $this->db->getInsertId();
		return $ret;
	}

	/**
	 * delete entry from database
	 */
	function delete(){
		if(!$this->ID){
			return false;
		}
		$this->db->query('DELETE FROM ' . $this->db->escape($this->table) . ' WHERE ID="' . intval($this->ID) . '"');
		return true;
	}

	/**
	 * check email syntax
	 */
	function check_email($email){
		return we_check_email($email);
	}

	/**
	 * check domain
	 */
	function check_domain($email, &$domain){
		$mxhosts = [];

		$domain = self::get_domain($email);
		if(!$domain){
			return false;
		}
		return (getmxrr(trim($domain), $mxhosts));
	}

	static function get_domain($email){
		$exp = "/[[:space:]\<_\.0-9A-Za-z-]+@([0-9a-zA-Z][0-9a-zA-Z-\.]+)(\>)?/";
		$out = [];
		if(preg_match_all($exp, $email, $out, PREG_PATTERN_ORDER)){
			return $out[1][0];
		}
		return false;
	}

	public static function getEmailsFromList($emails, $emails_only = 0, $group = 0, array $blocks = []){
		$arr = array_filter(explode("\n", $emails));
		if(!$arr){
			return [];
		}
		$ret = [];
		$default_html = f('SELECT pref_value FROM ' . SETTINGS_TABLE . ' WHERE tool="newsletter" AND pref_name="default_htmlmail"');

		foreach($arr as $row){
			$arr2 = explode(',', $row);
			if(!$arr2){
				continue;
			}
			$ret[] = ($emails_only ?
				$arr2[0] : [
				$arr2[0],
				(isset($arr2[1]) && trim($arr2[1]) != '') ? trim($arr2[1]) : $default_html,
				isset($arr2[2]) ? trim($arr2[2]) : '',
				isset($arr2[3]) ? $arr2[3] : '',
				isset($arr2[4]) ? $arr2[4] : '',
				isset($arr2[5]) ? $arr2[5] : '',
				$group,
				$blocks
			]);
		}

		return $ret;
	}

	public static function getEmailsFromExtern($files, $emails_only = 0, $group = 0, array $blocks = []){
		$arr = array_filter(explode(',', $files));
		if(!$arr){
			return [];
		}
		$default_html = f('SELECT pref_value FROM ' . SETTINGS_TABLE . ' WHERE tool="newsletter" AND pref_name="default_htmlmail"');
		$ret = [];
		foreach($arr as $file){
			if(strpos($file, '..') === false){
				$dataArr = explode("\n", str_replace("\r\n", "\n", we_base_file::load($_SERVER['DOCUMENT_ROOT'] . $file)));
				foreach($dataArr as $value){
					$dat = explode(',', $value);
					if(str_replace(' ', '', implode('', $dat)) === ''){
						continue;
					}
					switch($emails_only){
						case 1:
							$ret[] = $dat[0];
							break;
						case 2:
							$ret[] = [
								trim($dat[0]),
								(isset($dat[1]) && trim($dat[1]) != '') ? trim($dat[1]) : $default_html,
								isset($dat[2]) ? trim($dat[2]) : '',
								isset($dat[3]) ? $dat[3] : '',
								isset($dat[4]) ? $dat[4] : '',
								isset($dat[5]) ? $dat[5] : ''
							];
							break;
						default:
							$ret[] = [
								trim($dat[0]),
								(isset($dat[1]) && trim($dat[1]) != '') ? trim($dat[1]) : $default_html,
								isset($dat[2]) ? trim($dat[2]) : '',
								isset($dat[3]) ? $dat[3] : '',
								isset($dat[4]) ? $dat[4] : '',
								isset($dat[5]) ? $dat[5] : '',
								$group,
								$blocks
							];
					}
				}
			}
		}

		return $ret;
	}

	/**
	 * Enter description here...
	 *
	 * @param unknown_type $files
	 * @param unknown_type $emails_only
	 * @param unknown_type $group
	 * @param unknown_type $blocks
	 * @param int $status (0=all; 1=invalid; 2=valid )
	 * @return array
	 */
	static function getEmailsFromExtern2($files, $status, &$emailkey){
		$arr = array_filter(explode(',', $files));
		if(!$arr){
			return [];
		}
		$ret = [];
		$countEMails = 0;
		$default_html = f('SELECT pref_value FROM ' . SETTINGS_TABLE . ' WHERE tool="newsletter" AND pref_name="default_htmlmail"');
		$saveTrimmed = ($status == we_newsletter_newsletter::CSV_STATUS_ALL);
		foreach($arr as $file){
			if(strpos($file, '..') === false){
				$dataArr = explode("\n", str_replace("\r\n", "\n", ($rawData = we_base_file::load($_SERVER['DOCUMENT_ROOT'] . $file))));
				foreach($dataArr as $value){
					$dat = explode(',', $value);
					if(str_replace(' ', '', implode('', $dat)) === ''){
						continue;
					}
					$countEMails++;
					if(($status == we_newsletter_newsletter::CSV_STATUS_INVALID && we_check_email($dat[0])) || ($status == we_newsletter_newsletter::CSV_STATUS_VALID && !we_check_email($dat[0]))){
						continue;
					}
					$emailkey[] = $countEMails - 1;

					$ret[] = [
						trim($dat[0]),
						(isset($dat[1]) && trim($dat[1]) != '') ? trim($dat[1]) : $default_html,
						isset($dat[2]) ? trim($dat[2]) : '',
						isset($dat[3]) ? trim($dat[3]) : '',
						isset($dat[4]) ? trim($dat[4]) : '',
						isset($dat[5]) ? trim($dat[5]) : ''
					];
				}
			}

			if($saveTrimmed){
				$content = '';
				foreach($ret as $entry){
					$content .= implode(',', array_slice($entry, 0, 6)) . "\n";
				}
				if($content !== $rawData){
					while(($lock = we_base_file::lock(basename($file))) == false){
						usleep(100000);
					}
					we_base_file::save($_SERVER['DOCUMENT_ROOT'] . $file, $content);
					we_base_file::unlock($lock);
				}
			}
		}

		return $ret;
	}

}
