<?php

/**
 * webEdition CMS
 *
 * $Rev$
 * $Author$
 * $Date$
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_search_frames extends we_modules_frame{
	const TAB_DOCUMENTS = 1;
	const TAB_TEMPLATES = 2;
	const TAB_MEDIA = 5;
	const TAB_ADVANCED = 3;
	const TAB_PROPERTIES = 4;

	public function __construct(){
		$this->module = we_modules_info::SEARCH;
		parent::__construct(WEBEDITION_DIR . 'we_showMod.php?mod=weSearch');
		$this->Tree = new we_search_tree($this->jsCmd);

		$model = empty($_SESSION['weS']['weSearch_session']) ? new we_search_model() : $_SESSION['weS']['weSearch_session'];
		$this->View = new we_search_view($model);
	}

	protected function getHTMLCmd(){
		if(($pid = we_base_request::_(we_base_request::INT, 'pid')) === false){
			return $this->getHTMLDocument(we_html_element::body(), $this->jsCmd->getCmds());
		}

		$offset = we_base_request::_(we_base_request::INT, 'offset', 0);
		$attr = [];

		if(!empty($_SESSION['weS']['weSearch']['modelidForTree'])){
			$attr['onload'] = "top.content.treeData.selectNode(top.content.treeData.table,'" . ($_SESSION['weS']['weSearch']['modelidForTree']) . "');";
			unset($_SESSION['weS']['weSearch']['modelidForTree']);
		}
		$this->jsCmd->addCmd('loadTree', ['clear' => !$pid, 'items' => we_search_tree::getItems($pid, $offset, $this->Tree->default_segment)]);

		return $this->getHTMLDocument(
				we_html_element::body($attr, we_html_element::form(['name' => 'we_form'
						], we_html_element::hiddens(['pnt' => 'cmd',
							'cmd' => 'no_cmd'])
		)));
	}

	protected function getHTMLEditorHeader($mode = 0){
		$we_tabs = new we_gui_tabs();

		if($this->View->Model->IsFolder == 0){
			$displayEntry = 'inline';
			$displayFolder = 'none';
		} else {
			//folders and entries have different tabs to display
			$displayEntry = 'none';
			$displayFolder = 'inline';
		}

		//tabs for entries
		if(we_base_permission::hasPerm(we_base_perm::WORK_DOCUMENTS_CAN_SEE)){
			$we_tabs->addTab(g_l('searchtool', '[documents]'), 'fa-file-o', false, self::TAB_DOCUMENTS, ['id' => 'tab_1', 'style' => 'display:' . $displayEntry]);
		}
		if($_SESSION['weS']['we_mode'] != we_base_constants::MODE_SEE && we_base_permission::hasPerm(we_base_perm::WORK_TEMPLATES_CAN_SEE)){
			$we_tabs->addTab(g_l('searchtool', '[templates]'), 'fa-file-code-o', false, self::TAB_TEMPLATES, ['id' => 'tab_2',
				'style' => 'display:' . $displayEntry]);
		}
		if(we_base_permission::hasPerm(we_base_perm::WORK_DOCUMENTS_CAN_SEE)){// FIXME: add some media related perm
			$we_tabs->addTab(g_l('searchtool', '[media]'), 'fa-image', false, self::TAB_MEDIA, ['id' => 'tab_5', 'style' => 'display:' . $displayEntry]);
		}
		$we_tabs->addTab(g_l('searchtool', '[advSearch]'), 'fa-search-plus', false, self::TAB_ADVANCED, ['id' => 'tab_3',
			'style' => 'display:' . $displayEntry]);

		//tabs for folders
		$we_tabs->addTab(g_l('searchtool', '[properties]'), '', false, self::TAB_PROPERTIES, ['id' => 'tab_4', 'style' => 'display:' . $displayFolder]);

		$tabNr = $this->getTab();
		$this->jsCmd->addCmd('setTab', $tabNr);

		$tabsHead = we_html_element::cssLink(CSS_DIR . 'we_tab.css') .
			we_html_element::jsScript(JS_DIR . 'initTabs.js') .
			we_html_element::jsScript(WE_JS_MODULES_DIR . 'weSearch/search_view2.js');

		$Text = we_search_model::getLangText($this->View->Model->Path, $this->View->Model->Text);
		$body = we_html_element::body(
				['id' => 'eHeaderBody',
				'onload' => "weTabs.setFrameSize();document.getElementById('tab_'+top.content.activ_tab).className='tabActive';",
				'onresize' => 'weTabs.setFrameSize()'
				], '<div id="main"><div id="headrow">&nbsp;' .
				we_html_element::b(g_l('searchtool', ($this->View->Model->IsFolder ? '[topDir]' : '[topSuche]')) . ':&nbsp;' . $Text . '<div id="mark"><i class="fa fa-asterisk modified"></i></div>') .
				'</div>' .
				$we_tabs->getHTML() .
				'</div>');

		return $this->getHTMLDocument($body, $tabsHead);
	}

	protected function getHTMLEditorBody(){
		$body = we_html_element::body(
				['class' => 'weEditorBody',
				'onkeypress' => 'if(event.keyCode==13 || event.keyCode==3){weSearch.search(true);}',
				'onload' => 'loaded=1;setTimeout(weSearch.init,200);',
				'onresize' => 'weSearch.sizeScrollContent();'
				], we_html_element::form(['name' => 'we_form',
					'onsubmit' => 'return false'
					], $this->getHTMLProperties() .
					we_html_element::hiddens(['predefined' => $this->View->Model->predefined,
						'savedSearchName' => $this->View->Model->Text
				]))
		);

		$whichSearch = we_search_view::SEARCH_DOCS;
		$tabNr = $this->getTab();

		switch($tabNr){
			case self::TAB_DOCUMENTS:
				$whichSearch = we_search_view::SEARCH_DOCS;
				break;
			case self::TAB_TEMPLATES:
				$whichSearch = we_search_view::SEARCH_TMPL;
				break;
			case self::TAB_ADVANCED:
				$whichSearch = we_search_view::SEARCH_ADV;
				break;
			case self::TAB_MEDIA:
				$whichSearch = we_search_view::SEARCH_MEDIA;
				break;
		}

		return $this->getHTMLDocument($body, $this->View->getJSProperty() . $this->View->getSearchJS($whichSearch) . we_html_element::jsScript(JS_DIR . 'multiIconBox.js'));
	}

	private function getTab(){
		$cmdid = we_base_request::_(we_base_request::INT, 'cmdid', '');
		if($cmdid != ''){
			$_REQUEST['searchstartAdvSearch'] = 0;
		}
		if(($tab = we_base_request::_(we_base_request::INT, 'tab'))){
			return $tab;
		}
		if($cmdid != ''){
			return $this->View->Model->activTab;
		}
		return we_base_request::_(we_base_request::INT, 'tabnr', self::TAB_DOCUMENTS);
	}

	private function getHTMLProperties($preselect = ''){
		$tabNr = $this->getTab();

		return $this->View->getCommonHiddens(['cmd' => '',
				'pnt' => 'edbody',
				'tabnr' => $tabNr,
			]) .
			we_html_element::hidden('newone', ($this->View->Model->ID == 0 ? 1 : 0)) .
			we_html_element::div(['id' => 'tab1', 'style' => ($tabNr == self::TAB_DOCUMENTS ? 'display: block;' : 'display: none')], $tabNr == self::TAB_DOCUMENTS ? $this->getHTMLSearchtool($this->getHTMLTabDocuments()) : '') .
			we_html_element::div(['id' => 'tab2', 'style' => ($tabNr == self::TAB_TEMPLATES ? 'display: block;' : 'display: none')], $tabNr == self::TAB_TEMPLATES ? $this->getHTMLSearchtool($this->getHTMLTabTemplates()) : '') .
			we_html_element::div(['id' => 'tab5', 'style' => ($tabNr == self::TAB_MEDIA ? 'display: block;' : 'display: none')], $tabNr == self::TAB_MEDIA ? $this->getHTMLSearchtool($this->getHTMLTabMedia()) : '') .
			we_html_element::div(['id' => 'tab3', 'style' => ($tabNr == self::TAB_ADVANCED ? 'display: block;' : 'display: none')], $tabNr == self::TAB_ADVANCED ? $this->getHTMLSearchtool($this->getHTMLTabAdvanced()) : '') .
			we_html_element::div(['id' => 'tab4', 'style' => ($tabNr == self::TAB_PROPERTIES ? 'display: block;' : 'display: none')], $this->getHTMLSearchtool($this->getHTMLGeneral()));
	}

	private function getHTMLGeneral(){
		$disabled = true;
		$this->View->Model->Text = we_search_model::getLangText($this->View->Model->Path, $this->View->Model->Text);

		return ['',
			[['headline' => g_l('searchtool', '[general]'),
				'html' => we_html_tools::formElementTable(g_l('searchtool', '[dir]'), we_html_tools::textInput('Text', $this->View->Model->Text, '', ['style' => "width: 520px"], '', '', '', '', $disabled)),
				'space' => we_html_multiIconBox::SPACE_MED,
				'noline' => true
				]],
		];
	}

	private function getHTMLTabDocuments(){
		//parameter: search of the tab (load only search dependent model data in the view)
		$innerSearch = we_search_view::SEARCH_DOCS;

		$content = $this->View->searchclass->searchProperties($innerSearch, $this->View->Model);
		$headline = $this->View->makeHeadLines($innerSearch);
		$foundItems = $_SESSION['weS']['weSearch']['foundItems' . $innerSearch];

		return [[
			['headline' => g_l('searchtool', '[text]'),
				'html' => '<div>' . $this->View->getSearchDialog($innerSearch) . '</div>',
				'icon' => we_html_multiIconBox::PROP_DOC,
				'noline' => true,
			],
			['headline' => g_l('searchtool', '[suchenIn]'),
				'html' => '<div>' . $this->View->getDirSelector($innerSearch) . '</div>',
				'icon' => we_html_multiIconBox::PROP_PATH,
				'noline' => true,
			],
			['headline' => g_l('searchtool', '[optionen]'),
				'html' => '<div>' . $this->View->getSearchDialogOptions($innerSearch) . '</div>',
				'icon' => we_html_multiIconBox::PROP_ATTRIB,
			],
			],
			[['headline' => '',
				'html' => '<div>
		<div id="parametersTop_' . $innerSearch . '">' . $this->View->getSearchParameterTop($foundItems, $innerSearch) . '</div>' . $this->View->tblList($content, $headline, $innerSearch) . '<div id="parametersBottom_' . $innerSearch . '">' . $this->View->getSearchParameterBottom($foundItems, $innerSearch) . '</div>
		</div>',
				'space' => we_html_multiIconBox::SPACE_MED]]
		];
	}

	private function getHTMLTabTemplates(){
		$innerSearch = we_search_view::SEARCH_TMPL;

		$content = $this->View->searchclass->searchProperties($innerSearch, $this->View->Model);
		$headline = $this->View->makeHeadLines($innerSearch);
		$foundItems = $_SESSION['weS']['weSearch']['foundItemsTmplSearch'];

		return [
			[['headline' => g_l('searchtool', '[text]'),
				'html' => '<div>' . $this->View->getSearchDialog($innerSearch) . '</div>',
				'icon' => we_html_multiIconBox::PROP_DOC,
				'noline' => true,
				],
				['headline' => g_l('searchtool', '[suchenIn]'),
					'html' => '<div>' . $this->View->getDirSelector($innerSearch) . '</div>',
					'icon' => we_html_multiIconBox::PROP_PATH,
					'noline' => true,
				],
				['headline' => g_l('searchtool', '[optionen]'),
					'html' => '<div>' . $this->View->getSearchDialogOptions($innerSearch) . '</div>',
					'icon' => we_html_multiIconBox::PROP_ATTRIB,
				],
			],
			[['headline' => '',
				'html' => '<div>
		<div id="parametersTop_' . $innerSearch . '">' . $this->View->getSearchParameterTop($foundItems, $innerSearch) . '</div>' .
				$this->View->tblList($content, $headline, $innerSearch) . '<div id="parametersBottom_TmplSearch">' . $this->View->getSearchParameterBottom($foundItems, $innerSearch) . '</div>
		</div>',
				'space' => we_html_multiIconBox::SPACE_MED]]
		];
	}

	private function getHTMLTabMedia(){
		$innerSearch = we_search_view::SEARCH_MEDIA;

		$content = $this->View->searchclass->searchProperties($innerSearch, $this->View->Model);
		$headline = $this->View->makeHeadLines($innerSearch);
		$foundItems = $_SESSION['weS']['weSearch']['foundItems' . $innerSearch];

		return [
			[['headline' => g_l('searchtool', '[text]'),
				'html' => '<div>' . $this->View->getSearchDialog($innerSearch) . '</div>',
				'icon' => we_html_multiIconBox::PROP_DOC,
				'noline' => true,
				],
				['headline' => g_l('searchtool', '[suchenIn]'),
					'html' => '<div>' . $this->View->getDirSelector($innerSearch) . '</div>',
					'icon' => we_html_multiIconBox::PROP_PATH,
					'noline' => true,
				],
				['headline' => g_l('searchtool', '[optionen]'),
					'html' => '<div>' . $this->View->getSearchDialogOptions($innerSearch) . '</div>',
					'icon' => we_html_multiIconBox::PROP_ATTRIB,
					'noline' => true,
				],
				['headline' => g_l('searchtool', '[anzeigen]'),
					'html' => '<div>' . $this->View->getSearchDialogMediaType($innerSearch) . '</div>',
					'icon' => we_html_multiIconBox::PROP_DISPLAY,
					'noline' => true,
				],
				['headline' => 'Filter', //g_l('searchtool', '[optionen]'),
					'html' => '<div>' . $this->View->getSearchDialogFilter($innerSearch) . '</div>',
					'icon' => we_html_multiIconBox::PROP_FILTER,
				],
			],
			[['headline' => '',
				'html' => '<div>
		<div id=\'parametersTop_' . $innerSearch . '\'>' . $this->View->getSearchParameterTop($foundItems, $innerSearch) . '</div>' . $this->View->tblList($content, $headline, $innerSearch) . '<div id=\'parametersBottom_' . $innerSearch . '\'>' . $this->View->getSearchParameterBottom($foundItems, $innerSearch) . '</div>
		</div>']]
		];
	}

	private function getHTMLTabAdvanced(){
		$content = $this->View->searchclass->searchProperties(we_search_view::SEARCH_ADV, $this->View->Model);
		$headline = $this->View->makeHeadLines(we_search_view::SEARCH_ADV);
		$foundItems = $_SESSION['weS']['weSearch']['foundItems' . we_search_view::SEARCH_ADV];

		return [
			[
				[//'headline' => g_l('searchtool', '[text]'),
					'html' => '<div>' . $this->View->getSearchDialogOptionalFields(we_search_view::SEARCH_ADV) . '</div>',
					'icon' => we_html_multiIconBox::PROP_DOC,
					'noline' => true,
				],
				['headline' => g_l('searchtool', '[anzeigen]'),
					'html' => '<div>' . $this->View->getSearchDialogCheckboxesAdvSearch() . '</div>',
					'icon' => we_html_multiIconBox::PROP_DISPLAY,
				],
			], [['headline' => '',
				'html' => '<div>
      <div id=\'parametersTop_' . we_search_view::SEARCH_ADV . '\'>' . $this->View->getSearchParameterTop($foundItems, we_search_view::SEARCH_ADV) . '</div>' .
				$this->View->tblList($content, $headline, we_search_view::SEARCH_ADV) . '<div id=\'parametersBottom_' . we_search_view::SEARCH_ADV . '\'>' . $this->View->getSearchParameterBottom($foundItems, we_search_view::SEARCH_ADV) . '</div>
      </div>',
				'space' => we_html_multiIconBox::SPACE_MED]]
		];
	}

	private function getHTMLSearchtool($content){
		return ($content[0] ? we_html_multiIconBox::getHTML('contentParameters', $content[0], true, '', 0, g_l('searchtool', '[suchen]'), g_l('searchtool', '[suchen]'), true, '', 'weSearch.sizeScrollContent();') : '') .
			we_html_multiIconBox::getHTML('contentResult', $content[1], true);
	}

	function getHTML($what = '', $mode = '', $step = 0){
		switch($what){
			case 'frameset':
				return $this->getHTMLFrameset('', ($tab = we_base_request::_(we_base_request::INT, 'tab')) !== false ? '&tab=' . $tab : '');
			case 'editor':
				return $this->getHTMLEditor(($tab = we_base_request::_(we_base_request::INT, 'tab')) !== false ? '&tab=' . $tab : '');
			case 'edfooter':
				return $this->getHTMLEditorFooter([we_html_button::FOOTER_SAVE => [[], 'weSearch_save']]);
			default:
				return parent::getHTML($what, $mode, $step);
		}
	}

	protected function getHTMLFrameset($extraHead = '', $extraUrlParams = ''){
		return parent::getHTMLFrameset($this->Tree->getJSTreeCode() . $extraHead, $extraUrlParams);
	}

}
