<?php

/**
 * webEdition CMS
 *
 * $Rev: fa498c312e65 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2018-08-29 23:14 +0000 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_tag_banner extends we_weTag_tag{

	public static function tag(array $attribs = [], $content = '', $internal = false){
		if(($foo = self::attributeMissing($attribs, 'name', __CLASS__))){
			return $foo;
		}

		$bannername = self::getAttribute('_name_orig', $attribs, '', we_base_request::STRING);
		$paths = self::getAttribute('paths', $attribs, '', we_base_request::RAW);
		$type = self::getAttribute('type', $attribs, 'js', we_base_request::STRING);
		$target = self::getAttribute('target', $attribs, '', we_base_request::STRING);
		$width = self::getAttribute('width', $attribs, ($type === "pixel") ? 1 : '', we_base_request::UNIT);
		$height = self::getAttribute('height', $attribs, ($type === 'pixel') ? 1 : '', we_base_request::UNIT);
		$link = self::getAttribute('link', $attribs, true, we_base_request::BOOL);
		$page = self::getAttribute('page', $attribs, '', we_base_request::RAW);
		$bannerclick = self::getAttribute('clickscript', $attribs, WEBEDITION_DIR . 'bannerclick.php', we_base_request::URL);
		$getbanner = self::getAttribute('getscript', $attribs, WEBEDITION_DIR . 'getBanner.php', we_base_request::URL);
		$xml = self::getAttribute('xml', $attribs, XHTML_DEFAULT, we_base_request::BOOL);

		$nocount = $GLOBALS['we']['main']['doc']->InWebEdition;
		$did = $GLOBALS['we']['main']['id']; //Fix #10759

		if($type === 'pixel'){

			$newAttribs['src'] = $getbanner . '?' .
				http_build_query([
					'nocount' => $nocount,
					'type' => 'pixel',
					'paths' => $paths,
					'bannername' => $bannername,
					'cats' => $GLOBALS['we']['main']['doc']->Category,
					'dt' => (empty($GLOBALS['we']['main']['doc']->DocType) ? '' : $GLOBALS['we']['main']['doc']->DocType),
					'page' => $page ?: 0,
					'did' => $page ? 0 : $did,
					'xml' => $xml ? 1 : 0,
			]);
			$newAttribs['border'] = 0;
			$newAttribs['alt'] = '';
			$newAttribs['width'] = 1;
			$newAttribs['height'] = 1;

			return getHtmlTag('img', $newAttribs);
		}

		$uniq = md5(uniqid(__CLASS__, true));

		// building noscript
		// here build image with link(opt)
		$imgAtts['src'] = $getbanner . '?' .
			http_build_query([
				'c' => 1,
				'bannername' => $bannername,
				'cats' => (empty($GLOBALS['we']['main']['doc']->Category) ? '' : $GLOBALS['we']['main']['doc']->Category ),
				'dt' => (empty($GLOBALS['we']['main']['doc']->DocType) ? '' : $GLOBALS['we']['main']['doc']->DocType),
				'paths' => $paths,
				'page' => $page ?: 0,
				'did' => $page ? 0 : $did,
				'bannerclick' => $bannerclick,
				'xml' => ($xml ? 1 : 0)
		]);
		$imgAtts['alt'] = '';
		$imgAtts['border'] = 0;
		if($width){
			$imgAtts['width'] = $width;
		}
		if($height){
			$imgAtts['height'] = $height;
		}
		$img = getHtmlTag('img', $imgAtts);

		if($link){ //  with link
			$linkAtts['href'] = $bannerclick . '?' . http_build_query([
					'nocount' => $nocount,
					'u' => $uniq,
					'bannername' => $bannername,
					'type' => $type,
					'page' => $page ?: 0,
					'did' => $page ? 0 : $did
			]);
			if($target){
				$linkAtts['target'] = $target;
			}
			$noscript = getHtmlTag('a', $linkAtts, $img);
		} else { //  only img
			$noscript = $img;
		}


		switch($type){
			case 'iframe':
				// stuff for iframe  and ilayer
				$newAttribs = removeAttribs($attribs, ['name', 'paths', 'type', 'target', 'link', 'clickscript', 'getscript', 'page']);
				$newAttribs['xml'] = $xml ? 'true' : 'false';
				$newAttribs['width'] = $width ?: 468;
				$newAttribs['height'] = $height ?: 60;
				$newAttribs['src'] = $getbanner . '?' . http_build_query([
						'nocount' => $nocount,
						'bannername' => $bannername,
						'cats' => $GLOBALS['we']['main']['doc']->Category,
						'link' => ($link ? 1 : 0),
						'type' => 'iframe',
						'page' => $page ?: 0,
						'did' => $page ? 0 : $did,
						'paths' => $paths,
						'target' => $target,
						'bannerclick' => $bannerclick,
						'width' => $width,
						'height' => $height,
						'xml' => ($xml ? '1' : '0')
				]);

				return getHtmlTag('iframe', $newAttribs, $noscript);
			default:
			case 'js':

				return ((isset($GLOBALS['we_obj']->ID) ? true : $GLOBALS['we']['main']['doc']->IsDynamic) ?
					we_banner_banner::getBannerCode($did, $paths, $target, $width, $height, [$GLOBALS['we']['main']['doc']->DocType], array_filter(explode(',', $GLOBALS['we']['main']['doc']->Category)), $bannername, $link, $bannerclick, $getbanner, '', $page, $GLOBALS['we']['main']['doc']->InWebEdition, $xml) :
					($type === 'cookie' ?
					$noscript :
					we_html_element::jsElement('
var r = Math.random(),
scr = document.createElement("script");
scr.setAttribute("src", ' . $getbanner . '?' . ($nocount ? 'nocount=' . $nocount . '&amp;' : '') . 'r="+r+"&amp;link=' . ($link ? 1 : 0) . '&amp;bannername=' . rawurlencode($bannername) . '&amp;type=js' . ($page ? ('&amp;page=' . rawurlencode($page)) : ('&amp;did=' . $did . '&amp;paths=' . rawurlencode($paths))) . '&amp;target=' . rawurlencode($target) . '&amp;bannerclick=' . rawurlencode($bannerclick) . '&amp;height=' . rawurlencode($height) . '&amp;width=' . rawurlencode($width) . '");
document.getElementsByTagName("head")[0].appendChild(scr);
') .
					'<noscript>' . $noscript . '</noscript>'
					)
					);
		}
	}

}

function we_tag_banner(array $attribs){
	return we_tag_banner::tag($attribs);
}
