<?php

/**
 * webEdition CMS
 *
 * $Rev: 27aac04455e2 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2018-08-27 10:09 +0000 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_tag_ifVarEmpty extends we_weTag_ifTag{

	public static function tag(array $attribs = [], $content = '', $internal = false){
		$name = self::getAttribute('name', $attribs, '', we_base_request::STRING);
		$orgName = self::getAttribute('_name_orig', $attribs, '', we_base_request::STRING);

		if(!$name){
			$attribs['_name_orig'] = $orgName = self::getAttribute('match', $attribs, '', we_base_request::STRING);
			$attribs['name'] = $name = self::getPostName($orgName);
		}

		if(!$orgName && ($foo = self::attributeMissing($attribs, 'name', __CLASS__))){
			echo $foo;
			return false;
		}
		$type = self::getAttribute('type', $attribs, '', we_base_request::STRING);

		switch($type){
			case 'request' :
				$ret = self::getArrayValue($_REQUEST, null, $orgName);
				return empty($ret);
			case 'post' :
				$ret = self::getArrayValue($_POST, null, $orgName);
				return empty($ret);
			case 'get' :
				$ret = self::getArrayValue($_GET, null, $orgName);
				return empty($ret);
			case 'global' :
				$ret = self::getArrayValue($GLOBALS, null, $orgName);
				return empty($ret);
			case 'session' :
				$ret = self::getArrayValue($_SESSION, null, $orgName);
				return empty($ret);
			case 'sessionfield' :
				return empty($_SESSION['webuser'][$orgName]);
		}

		$docAttr = self::getAttribute('doc', $attribs, '', we_base_request::STRING);
		switch($docAttr){
			case 'object' :
			case 'document' :
				$formname = self::getAttribute('formname', $attribs, we_base_constants::DEFAULT_FORM, we_base_request::STRING);
				$doc = isset($GLOBALS['we_' . $docAttr][$formname]) ? $GLOBALS['we_' . $docAttr][$formname] : false;
				break;
			case 'top':
				$doc = empty($GLOBALS['we']['main']['doc']) ? false : $GLOBALS['we']['main']['doc'];
				break;
			default:
				$doc = isset($GLOBALS['we_doc']) ? $GLOBALS['we_doc'] : false;
		}
		if(!$doc){
			return true;
		}

		if(self::getAttribute('property', $attribs, false, we_base_request::BOOL)){
			return empty($doc->$orgName);
		}

		$type = $type ?: $doc->getElement($orgName, 'type');
		switch($type){
			case 'href' :
				$foo = $doc->getField($attribs, $type, true);
				return empty($foo) || $foo === '/';
			case 'multiobject':
				//FIXME: this makes no sense
				/* $attribs['name'] = $name;
				  $attribs['_name_orig'] = $orgName; */
				$data = we_unserialize($doc->getField($attribs, $type, true));
				$objects = isset($data['objects']) ? $data['objects'] : $data;
				return empty($objects);
			default:
				$elemType = $doc->getElement($name, 'type');
				$foo = $doc->getElement($name, $elemType === 'img' ? we_contents_base::ETYPE_NUMBER : we_contents_base::ETYPE_UNKNOWN);

				if(!$foo){
					$elemType = $doc->getElement($orgName, 'type');
					$foo = $doc->getElement($orgName, $elemType === 'img' ? we_contents_base::ETYPE_NUMBER : we_contents_base::ETYPE_UNKNOWN);
				}
				return empty($foo);
		}
	}

}

function we_tag_ifVarEmpty(array $attribs){
	return we_tag_ifVarEmpty::tag($attribs);
}
