<?php

/**
 * webEdition CMS
 *
 * $Rev: ab81cbaac3fa $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2018-08-28 09:41 +0000 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_tag_sendMail extends we_weTag_tag{

	public static function tag(array $attribs = [], $content = '', $internal = false){
		if(($foo = self::attributeMissing($attribs, ['recipient' => false, 'from' => false], __CLASS__))){
			return $foo;
		}

		if($GLOBALS['we_doc']->InWebEdition){
			return '';
		}

		$id = self::getAttribute('id', $attribs, we_base_request::_(we_base_request::INT, 'ID', 0), we_base_request::INT);
		if(!$id){
			return '';
		}

		$from = self::getAttribute('from', $attribs, '', we_base_request::EMAIL);
		$reply = self::getAttribute('reply', $attribs, [], we_base_request::EMAILLISTA);
		$recipient = self::getAttribute('recipient', $attribs, '', we_base_request::STRING); //FIXME:email_list
		$recipientCC = self::getAttribute('recipientcc', $attribs, self::getAttribute('recipientCC', $attribs, '', we_base_request::STRING), we_base_request::STRING, true); //FIXME:email_list
		$recipientBCC = self::getAttribute('recipientbcc', $attribs, self::getAttribute('recipientBCC', $attribs, '', we_base_request::STRING), we_base_request::STRING, true); //FIXME:email_list

		$mimetype = self::getAttribute('mimetype', $attribs, '', we_base_request::STRING);
		$subject = self::getAttribute('subject', $attribs, '', we_base_request::STRING);
		$charset = self::getAttribute('charset', $attribs, 'UTF-8', we_base_request::STRING);
		$includeimages = self::getAttribute('includeimages', $attribs, false, we_base_request::BOOL);
		$useBaseHref = self::getAttribute('usebasehref', $attribs, true, we_base_request::BOOL);
		$useFormmailLog = self::getAttribute('useformmaillog', $attribs, false, we_base_request::BOOL);
		$useFormmailBlock = self::getAttribute('useformmailblock', $attribs, false, we_base_request::BOOL);
		if($useFormmailBlock){
			$useFormmailLog = true;
		}
		$blocked = false;

		$we_recipient = self::getMails(explode(',', $recipient));
		$we_recipientCC = self::getMails(explode(',', $recipientCC));
		$we_recipientBCC = self::getMails(explode(',', $recipientBCC));

		if($useFormmailLog){
			// insert into log
			$GLOBALS['DB_WE']->query('INSERT INTO ' . FORMMAIL_LOG_TABLE . ' SET IP="' . $GLOBALS['DB_WE']->escape($_SERVER['REMOTE_ADDR']) . '"');
			if(FORMMAIL_EMPTYLOG > -1){
				$GLOBALS['DB_WE']->query('DELETE FROM ' . FORMMAIL_LOG_TABLE . ' WHERE unixTime<(NOW()-INTERVAL ' . intval(FORMMAIL_EMPTYLOG) . ' SECOND)');
			}

			if($useFormmailBlock){
				$trials = FORMMAIL_TRIALS;
				// first delete all entries from blocktable which are older then now - blocktime
				$GLOBALS['DB_WE']->query('DELETE FROM ' . FORMMAIL_BLOCK_TABLE . ' WHERE blockedUntil!=-1 AND blockedUntil<UNIX_TIMESTAMP()');

				// check if ip is allready blocked
				if(f('SELECT 1 FROM ' . FORMMAIL_BLOCK_TABLE . ' WHERE ip="' . $GLOBALS['DB_WE']->escape($_SERVER['REMOTE_ADDR']) . '" LIMIT 1')){
					$blocked = true;
				} else {
					// ip is not blocked, so see if we need to block it
					if(f('SELECT COUNT(1) FROM ' . FORMMAIL_LOG_TABLE . ' WHERE unixTime>(NOW()- INTERVAL ' . intval(FORMMAIL_SPAN) . ' SECOND) AND ip="' . $GLOBALS['DB_WE']->escape($_SERVER['REMOTE_ADDR']) . '"') > $trials){
						$blocked = true;
						// insert in block table
						$blockedUntil = (FORMMAIL_BLOCKTIME == -1) ? -1 : '(UNIX_TIMESTAMP()+' . FORMMAIL_BLOCKTIME . ')';
						$GLOBALS['DB_WE']->query('REPLACE INTO ' . FORMMAIL_BLOCK_TABLE . " (ip, blockedUntil) VALUES('" . $GLOBALS['DB_WE']->escape($_SERVER['REMOTE_ADDR']) . "', " . $blockedUntil . ")");
					}
				}
			}
		}
		if($blocked){
			$headline = 'Fehler / Error';
			$content = g_l('global', '[formmailerror]') . getHtmlTag('br') . '&#8226; ' . 'Email dispatch blocked / Email Versand blockiert!';

			echo we_html_tools::getHtmlTop('', '', '', '', getHtmlTag('body', ['class' => 'weEditorBody'], we_html_tools::dialogLayout(getHtmlTag('div', ['class' => 'defaultfont lowContrast'], $content), $headline)));

			exit;
		}
		if(!isset($_SESSION)){
			new we_base_sessionHandler();
		}
		$GLOBALS['we']['WE_SendMail'] = true;
		$codes = we_base_file::isWeFile($id, FILE_TABLE, $GLOBALS['DB_WE']) ? we_getDocumentByID($id) : '';
		unset($GLOBALS['we']['WE_SendMail']);
		if(!$codes){
			t_e('Document to send via we:sendMail is empty ID: ' . $id);
		}
		$phpmail = new we_mail_mail($charset, $we_recipient, $subject, $from, $reply, $includeimages);
		if(isset($includeimages)){
			$phpmail->setIsEmbedImages($includeimages);
		}
		if(($we_recipientCC)){
			$phpmail->setCC($we_recipientCC);
		}
		if(($we_recipientBCC)){
			$phpmail->setBCC($we_recipientBCC);
		}
		if(isset($useBaseHref)){
			$phpmail->setIsUseBaseHref($useBaseHref);
		}
		if($mimetype != we_mail_mime::TYPE_HTML){
			$phpmail->setTextPartOutOfHTML($codes);
		} else {
			$phpmail->addHTMLPart($codes);
		}
		$phpmail->buildMessage();
		$phpmail->send();
		return '';
	}

	private static function getMails($to){
		$we_recipient = [];
		foreach($to as $mail){
			if(strpos($mail, '@') === false){
				if((
					(!empty($_SESSION['webuser']['registered'])) ||
					(we_tag_customerResetPassword::$status === we_customer_customer::PWD_ALL_OK)) &&
					isset($_SESSION['webuser'][$mail]) && strpos($_SESSION['webuser'][$mail], '@') !== false){ //wenn man registireten Usern was senden moechte
					if(we_check_email($_SESSION['webuser'][$mail])){
						$we_recipient[] = $_SESSION['webuser'][$mail];
					}
				} else if(isset($_REQUEST[$mail]) && strpos($_REQUEST[$mail], '@') !== false){ //email to friend test
					if(we_check_email(($mail = we_base_request::_(we_base_request::EMAIL, $mail)))){
						$we_recipient[] = $mail;
					}
				}
			} else {
				if(we_check_email($mail)){
					$we_recipient[] = $mail;
				}
			}
		}

		return $we_recipient;
	}

}

function we_tag_sendMail(array $attribs){
	return we_tag_sendMail::tag($attribs);
}
